/*******************************************************************************************/
/**                                                                                       **/
/** COPYRIGHT (C) 2019, Systemyde International Corporation, ALL RIGHTS RESERVED          **/
/**                                                                                       **/
/** 1024x8 fifo module                                                Rev 0.0  09/23/2019 **/
/**                                                                                       **/
/*******************************************************************************************/
module fifo2_fpga (data_out, fifo_empt, fifo_full, fifo_ov, fill_levl, clk, data_in,
                   fifo_purge, fifo_rd, fifo_wr, reset);

  input         clk;           /* clock                                                    */
  input         fifo_purge;    /* fifo purge                                               */
  input         fifo_rd;       /* fifo read enable                                         */
  input         fifo_wr;       /* fifo write enable                                        */
  input         reset;         /* master reset                                             */
  input   [7:0] data_in;       /* fifo data input                                          */

  output        fifo_empt;     /* fifo empty                                               */
  output        fifo_full;     /* fifo full                                                */
  output        fifo_ov;       /* fifo overflowed                                          */
  output  [7:0] data_out;      /* fifo data output                                         */
  output [10:0] fill_levl;     /* fifo fill level                                          */

  /*****************************************************************************************/
  /* signal declarations                                                                   */
  /*****************************************************************************************/
  wire          fifo_empt;                                 /* fifo empty                   */
  wire          fifo_full;                                 /* fifo full                    */
  wire          fifo_rdl,   fifo_rdu;                      /* fifo read                    */
  wire          fifo_wrl,   fifo_wru;                      /* fifo write                   */
  wire    [7:0] data_outl,  data_outu;                     /* fifo data output             */

  reg           fifo_ov;                                   /* fifo overflowed              */
  reg           raddr9_dly;                                /* latched rd addr bit 9        */
  reg    [10:0] fill_levl;                                 /* fifo fill level              */
  reg    [10:0] raddr_reg;                                 /* read pointer                 */
  reg    [10:0] waddr_reg;                                 /* write pointer                */

  /*****************************************************************************************/
  /* read pointer                                                                          */
  /*****************************************************************************************/
  always @ (posedge clk or posedge reset) begin
    if (reset) begin
      raddr_reg <= 11'h0;
      end
    else if (fifo_purge || (fifo_rd && !fifo_empt)) begin
      raddr_reg <= (fifo_purge) ? 11'h0 : raddr_reg + 1'b1;
      end
    end

  /*****************************************************************************************/
  /* write pointer                                                                         */
  /*****************************************************************************************/
  always @ (posedge clk or posedge reset) begin
    if (reset) begin
      waddr_reg <= 11'h0;
      end
    else if (fifo_purge || (fifo_wr && !fifo_full)) begin
      waddr_reg <= (fifo_purge) ? 11'h0 : waddr_reg + 1'b1;
      end
    end

  /*****************************************************************************************/
  /* fifo status                                                                           */
  /*****************************************************************************************/
  assign fifo_empt = (raddr_reg == waddr_reg);
  assign fifo_full = (raddr_reg[9:0] == waddr_reg[9:0]) && (raddr_reg[10] ^ waddr_reg[10]);

  always @ (posedge clk or posedge reset) begin
    if (reset) begin
      fifo_ov   <= 1'b0;
      fill_levl <= 11'h0;
      end
    else begin
      fifo_ov   <= !fifo_purge && ((fifo_wr && fifo_full) || fifo_ov);
      fill_levl <= (waddr_reg - raddr_reg);
      end
    end

  /*****************************************************************************************/
  /* lower/upper multiplexing                                                              */
  /*****************************************************************************************/
  always @ (posedge clk or posedge reset) begin
    if        (reset) raddr9_dly <= 1'b0;
    else if (fifo_rd) raddr9_dly <= raddr_reg[9];
    end

  assign data_out = (raddr9_dly) ? data_outu : data_outl;
  assign fifo_wrl = fifo_wr && !waddr_reg[9];
  assign fifo_wru = fifo_wr &&  waddr_reg[9];

  /*****************************************************************************************/
  /* ram                                                                                   */
  /*****************************************************************************************/
`ifdef SIM_VERSION

  ram_512x8   RAM_FL ( .RADDR(raddr_reg[8:0]), .RCLK(clk), .RCLKE(fifo_rd), .RE(fifo_rd),
                       .RDATA(data_outl), .WADDR(waddr_reg[8:0]), .WCLK(clk),
                       .WCLKE(fifo_wrl), .WE(fifo_wrl), .WDATA(data_in) );

  ram_512x8   RAM_FU ( .RADDR(raddr_reg[8:0]), .RCLK(clk), .RCLKE(fifo_rd), .RE(fifo_rd),
                       .RDATA(data_outu), .WADDR(waddr_reg[8:0]), .WCLK(clk),
                       .WCLKE(fifo_wru), .WE(fifo_wru), .WDATA(data_in) );
`else

  SB_RAM512x8 RAM_FL ( .RADDR(raddr_reg[8:0]), .RCLK(clk), .RCLKE(fifo_rd), .RE(fifo_rd),
                       .RDATA(data_outl), .WADDR(waddr_reg[8:0]), .WCLK(clk),
                       .WCLKE(fifo_wrl), .WE(fifo_wrl), .WDATA(data_in) );

  SB_RAM512x8 RAM_FU ( .RADDR(raddr_reg[8:0]), .RCLK(clk), .RCLKE(fifo_rd), .RE(fifo_rd),
                       .RDATA(data_outu), .WADDR(waddr_reg[8:0]), .WCLK(clk),
                       .WCLKE(fifo_wru), .WE(fifo_wru), .WDATA(data_in) );
`endif

  endmodule





