/*******************************************************************************************/
/**                                                                                       **/
/** COPYRIGHT (C) 2018, Systemyde International Corporation, ALL RIGHTS RESERVED          **/
/**                                                                                       **/
/** 512x8 fifo module                                                 Rev 0.0  08/02/2018 **/
/**                                                                                       **/
/*******************************************************************************************/
module fifo_fpga (data_out, fifo_empt, fifo_full, fifo_ov, fill_levl, clk, data_in,
                  fifo_purge, fifo_rd, fifo_wr, reset);

  input        clk;            /* clock                                                    */
  input        fifo_purge;     /* fifo purge                                               */
  input        fifo_rd;        /* fifo read enable                                         */
  input        fifo_wr;        /* fifo write enable                                        */
  input        reset;          /* master reset                                             */
  input  [7:0] data_in;        /* fifo data input                                          */

  output       fifo_empt;      /* fifo empty                                               */
  output       fifo_full;      /* fifo full                                                */
  output       fifo_ov;        /* fifo overflowed                                          */
  output [7:0] data_out;       /* fifo data output                                         */
  output [9:0] fill_levl;      /* fifo fill level                                          */

  /*****************************************************************************************/
  /* signal declarations                                                                   */
  /*****************************************************************************************/
  wire        fifo_empt;                                   /* fifo_empty                   */
  wire        fifo_full;                                   /* fifo_full                    */
  wire  [7:0] data_out;                                    /* fifo data output             */

  reg         fifo_ov;                                     /* fifo_overflowed              */
  reg   [9:0] fill_levl;                                   /* fifo fill level              */
  reg   [9:0] raddr_reg;                                   /* read pointer                 */
  reg   [9:0] waddr_reg;                                   /* write pointer                */

  /*****************************************************************************************/
  /* read pointer                                                                          */
  /*****************************************************************************************/
  always @ (posedge clk or posedge reset) begin
    if (reset) begin
      raddr_reg <= 10'h0;
      end
    else if (fifo_purge || (fifo_rd && !fifo_empt)) begin
      raddr_reg <= (fifo_purge) ? 10'h0 : raddr_reg + 1'b1;
      end
    end

  /*****************************************************************************************/
  /* write pointer                                                                         */
  /*****************************************************************************************/
  always @ (posedge clk or posedge reset) begin
    if (reset) begin
      waddr_reg <= 10'h0;
      end
    else if (fifo_purge || (fifo_wr && !fifo_full)) begin
      waddr_reg <= (fifo_purge) ? 10'h0 : waddr_reg + 1'b1;
      end
    end

  /*****************************************************************************************/
  /* fifo status                                                                           */
  /*****************************************************************************************/
  assign fifo_empt = (raddr_reg == waddr_reg);
  assign fifo_full = (raddr_reg[8:0] == waddr_reg[8:0]) && (raddr_reg[9] ^ waddr_reg[9]);

  always @ (posedge clk or posedge reset) begin
    if (reset) begin
      fifo_ov   <= 1'b0;
      fill_levl <= 10'h0;
      end
    else begin
      fifo_ov   <= !fifo_purge && ((fifo_wr && fifo_full) || fifo_ov);
      fill_levl <= (waddr_reg - raddr_reg);
      end
    end

  /*****************************************************************************************/
  /* ram                                                                                   */
  /*****************************************************************************************/
`ifdef SIM_VERSION

  ram_512x8    RAM_F    ( .RADDR(raddr_reg[8:0]), .RCLK(clk), .RCLKE(fifo_rd), .RE(fifo_rd),
                          .RDATA(data_out), .WADDR(waddr_reg[8:0]), .WCLK(clk),
                          .WCLKE(fifo_wr), .WE(fifo_wr), .WDATA(data_in) );
`else

  SB_RAM512x8  RAM_F    ( .RADDR(raddr_reg[8:0]), .RCLK(clk), .RCLKE(fifo_rd), .RE(fifo_rd),
                          .RDATA(data_out), .WADDR(waddr_reg[8:0]), .WCLK(clk),
                          .WCLKE(fifo_wr), .WE(fifo_wr), .WDATA(data_in) );
`endif

  endmodule





