/*******************************************************************************************/
/**                                                                                       **/
/** COPYRIGHT (C) 2019,  Systemyde International Corporation, ALL RIGHTS RESERVED         **/
/**                                                                                       **/
/** gnss interface core                                               Rev 0.0  09/24/2019 **/
/**                                                                                       **/
/*******************************************************************************************/
module gnss_fpga (data_drv, data_out, debug_out, enabl_rd, enabl_wr, fi_drv, fi_out, gnss_en,
                  isa_drv, isa_out, osc_en, page_addr, tx_dout, clk, data_bus, debug_ctl,
                  isa_bus, ph1, ph2, pwr_upb, pwob, rx_din, sync);

  input         clk;           /* clk (2.4576MHz)                                     GB   */
  input         data_bus;      /* memory/peripheral data bus                          bus  */
  input         isa_bus;       /* instruction address/data bus                        bus  */
  input         ph1;           /* clock phase 1                                       GB   */
  input         ph2;           /* clock phase 2                                       GB   */
  input         pwr_upb;       /* power on reset (inverted)                           GB   */
  input         pwob;          /* power on (inverted)                                 GB   */
  input         rx_din;        /* rx data from gnss                                   ffc  */
  input         sync;          /* synchronization                                     bus  */
  input   [2:0] debug_ctl;     /* debug select signals                                dbg  */

  output        data_drv;      /* memory/peripheral data bus enable                   bus  */
  output        data_out;      /* memory/peripheral data bus                          bus  */
  output        enabl_rd;      /* imem read enable                                   imem  */
  output        enabl_wr;      /* imem write enable                                  imem  */
  output        fi_drv;        /* flag bus enable                                     bus  */
  output        fi_out;        /* flag bus                                            bus  */
  output        isa_drv;       /* instruction address/data bus enable                 bus  */
  output        isa_out;       /* instruction address/data bus                        bus  */
  output        gnss_en;       /* gnss enable                                         ffc  */
  output        osc_en;        /* oscillator enable                                   osc  */
  output        tx_dout;       /* tx data to gnss                                     ffc  */
  output  [3:0] page_addr;     /* imem page address                                  imem  */
  output  [8:0] debug_out;     /* debug output signals                                dbg  */

  /*****************************************************************************************/
  /* signal declarations                                                                   */
  /*****************************************************************************************/
  wire        auto_trig;                                   /* clocked serial auto trigger  */
  wire        clr_ncnt;                                    /* clear NMEA string counters   */
  wire        clr_nmea;                                    /* clear NMEA string id         */
  wire        data_drv;                                    /* data bus drive enable        */
  wire        data_out;                                    /* memory/peripheral data bus   */
  wire        en_scr1, en_scr2;                            /* scr recirculate enable       */
  wire        enabl_rd;                                    /* imem read enable             */
  wire        enabl_wr;                                    /* imem write enable            */
  wire        fi_drv;                                      /* flag bus drive enable        */
  wire        fi_out;                                      /* flag bus data                */
  wire        gnss_en;                                     /* gnss enable                  */
  wire        gsa_mode2;                                   /* mode2 value in gsa string    */
  wire        i_rm_addr;                                   /* inc rm read addr             */
  wire        inc_gsa, inc_gsv, inc_unk;                   /* inc NMEA string counters     */
  wire        ibd_en;                                      /* ibd read pls enable          */
  wire        isa_drv;                                     /* isa_bus drive                */
  wire        isa_out;                                     /* isa_bus data                 */
  wire        ld_cycle;                                    /* next cycle                   */
  wire        ld_gcs,  ld_rc,   ld_sbcs;                   /* load reg for read            */
  wire        ld_ibcs, ld_ibd,  ld_obcs;
  wire        ld_inst;                                     /* latch instruction            */
  wire        ld_rm_addr;                                  /* ld rm read addr              */
  wire        ld_rxlim;                                    /* ld rx length limit           */
  wire        ld_sblast;                                   /* ld sbuf last wr claim        */
  wire        ld_sbrsel;                                   /* ld sbuf read select          */
  wire        ld_sbwa;                                     /* load sbuf wr address         */
  wire        ld_sbwsel;                                   /* ld sbuf write select         */
  wire        ld_strtrk;                                   /* ld string comma track sm     */
  wire        nmea_end;                                    /* terminate nmea string rx     */
  wire        obd_en;                                      /* obd wr pls enable            */
  wire        osc_en;                                      /* oscillator enable            */
  wire        ph1;                                         /* buffered ph1                 */
  wire        ph2;                                         /* buffered ph2                 */
  wire        pwr_upb;                                     /* pwr_up (inverted)            */
  wire        pwob;                                        /* power on (inverted)          */
  wire        rdsbcs_inst;                                 /* sbcs read inst               */
  wire        rdsbd_inst;                                  /* sbd read inst                */
  wire        rdgcs_inst;                                  /* gcs read inst                */
  wire        rdrc_inst;                                   /* rc read inst                 */
  wire        rdibcs_inst;                                 /* ibcs read inst               */
  wire        rdibd_inst;                                  /* ibd read inst                */
  wire        rdobcs_inst;                                 /* obcs read inst               */
  wire        rdobd_inst;                                  /* obd read inst                */
  wire        rdscr1_inst;                                 /* scr1 read inst               */
  wire        rdscr2_inst;                                 /* scr2 read inst               */
  wire        rm_ld, rm_pld;                               /* rm read data load            */
  wire        rm_rd, rm_prd;                               /* rm read data read            */
  wire        rmc_valid;                                   /* valid value in rmc string    */
  wire        rx_1byte, rx_2byte;                          /* rx byte count for read       */
  wire        rx_3byte, rx_4byte;
  wire        rx_din_m;                                    /* muxed rx_din                 */
  wire        rx_empt;                                     /* rx fifo empty                */
  wire        rx_fifo_rd;                                  /* rx fifo read pulse           */
  wire        rx_fifo_wr;                                  /* rx fifo write pulse          */
  wire        rx_full;                                     /* rx byte available            */
  wire        rx_ov;                                       /* rx fifo overflow             */
  wire        rx_purge;                                    /* rx fifo purge                */
  wire        rx_read;                                     /* rd rx fifo                   */
  wire        rxmode_clr;                                  /* clear rxmode latches         */
  wire        sb00_rd, sb01_rd, sb02_rd;                   /* sb0 read strobes             */
  wire        sb10_rd, sb11_rd, sb12_rd;                   /* sb1 read strobes             */
  wire        sb00_wr, sb01_wr, sb02_wr;                   /* sb0 write strobes            */
  wire        sb10_wr, sb11_wr, sb12_wr;                   /* sb1 write strobes            */
  wire        sblast_nxt;                                  /* sbuf last wr next            */
  wire        sbr_ainc;                                    /* inc sb read addr             */
  wire        sbr_ld, sbr_pld;                             /* sb read data load            */
  wire        sbr_rd, sbr_prd;                             /* sb read data read            */
  wire        sbrd_clm;                                    /* string buffer rd claim       */
  wire        sbrd_rel;                                    /* string buffer rd release     */
  wire        sbw_wr;                                      /* sb write data write          */
  wire        sbwr_clm;                                    /* sbuf write claim             */
  wire        sbwr_rel;                                    /* sbuf write release           */
  wire        scr1_nxt, scr2_nxt;                          /* scr data in                  */
  wire        set_rx_gsa, set_rx_gsv;                      /* set NMEA string identifiers  */
  wire        set_rx_rmc, set_rx_unk;
  wire        set_sb0_inv, set_sb1_inv;                    /* set sb invalid global        */
  wire        set_sb0val,  set_sb1val;                     /* set sb valid global          */
  wire        srvc_drv;                                    /* service request enable       */
  wire        srvc_req;                                    /* service request              */
  wire        state_ctl;                                   /* states ec8012                */
  wire        state_byte;                                  /* states c02a96b               */
  wire        str_comma;                                   /* "," in string                */
  wire        tx_dout;                                     /* tx data to gnss              */
  wire        tx_empt;                                     /* tx fifo empty                */
  wire        tx_empty;                                    /* tx shifter empty             */
  wire        tx_fifo_rd;                                  /* tx fifo read pulse           */
  wire        tx_fifo_wr;                                  /* tx fifo write pulse          */
  wire        tx_full;                                     /* tx fifo full                 */
  wire        tx_purge;                                    /* tx fifo purge                */
  wire        tx_write;                                    /* tx write                     */
  wire        wribcs_inst;                                 /* ibcs write inst              */
  wire        wrgcs_inst;                                  /* gcs write inst               */
  wire        wrobcs_inst;                                 /* obcs write inst              */
  wire        wrobd_inst;                                  /* obd write inst               */
  wire        wrrc_inst;                                   /* rc write inst                */
  wire        wrsbc_inst;                                  /* sbc write inst               */
  wire        wrsbcs_inst;                                 /* sbcs write inst              */
  wire        wrsbr_inst;                                  /* sbr write inst               */
  wire        wrscr1_inst;                                 /* scr1 write inst              */
  wire        wrscr2_inst;                                 /* scr2 write inst              */
  wire  [2:0] rx_count;                                    /* rx byte count for status     */
  wire  [2:0] talk_id;                                     /* talker identifier            */
  wire  [3:0] page_addr;                                   /* imem page address            */
  wire  [7:0] ibd_nxt;                                     /* ibd load data                */
  wire  [7:0] rm_nxt;                                      /* reg mem load data            */
  wire  [7:0] rx_data;                                     /* rx fifo data in              */
  wire  [7:0] rx_fifo_out;                                 /* rx fifo data out             */
  wire  [7:0] sb00_dout, sb01_dout, sb02_dout;             /* sb0 out data                 */
  wire  [7:0] sb10_dout, sb11_dout, sb12_dout;             /* sb1 out data                 */
  wire  [7:0] tx_data;                                     /* fifo-tx data                 */
  wire  [8:0] rm_addr_mux;                                 /* reg mem address              */
  wire  [8:0] rxlim_nxt;                                   /* rx length limit counter      */
  wire  [9:0] rx_levl;                                     /* rx fifo level                */
  wire [10:0] tx_levl;                                     /* tx fifo level                */
  wire [10:0] sbradd_mux;                                  /* string bufr read addr        */
  wire [11:0] sbr_valid;                                   /* sb read valid                */
  wire [11:0] set_sb0_val, set_sb1_val;                    /* set sb valid individual      */

  reg         attn_reg;                                    /* clk-side interrupt           */
  reg         cregn_reg,  regnc_reg;                       /* latched inst decodes         */
  reg         daddc_reg,  pfadc_reg;                       /* dadd=c, pfad=c               */
  reg         data_reg;                                    /* ph1 sampled data signal      */
  reg         disrom_reg;                                  /* rom disable                  */
  reg         enabl_reg;                                   /* f1 chip enable               */
  reg         enram_reg;                                   /* f2-f5 chip enable            */
  reg         enrom_inv_reg;                               /* rom enable    (inv polarity) */
  reg         hldtx_reg;                                   /* hold transmit                */
  reg         iball_reg;                                   /* all rx to inbound            */
  reg         inten_reg;                                   /* interrupt enable             */
  reg         isa_reg;                                     /* ph1 sampled isa signal       */
  reg         powoff_inv_reg;                              /* powoff inst   (inv polarity) */
  reg         pus_inv_reg;                                 /* pwr-up state  (inv polarity) */
  reg         rm_wr;                                       /* rm write                     */
  reg         rmcval_reg;                                  /* rmc "valid" state            */
  reg         rwrom_reg;                                   /* rom read/write               */
  reg         rx_gsa_reg, rx_gsv_reg;                      /* NMEA state bits              */
  reg         rx_rmc_reg, rx_unk_reg;
  reg         rxen_reg;                                    /* rx enable                    */
  reg         rxst_cdc;                                    /* rx status trig               */
  reg         rx_idle;                                     /* rx idle line                 */
  reg         rx_limit;                                    /* rx length limit reached      */
  reg         sblast_reg;                                  /* sbuf last wr next            */
  reg         sbwclm_reg;                                  /* sb wr claim registered       */
  reg         ser_pls;                                     /* serial clock (16x rate)      */
  reg         state_0, state_1, state_2, state_3;          /* machine state                */
  reg         state_4, state_5, state_6;
  reg         state_8, state_9, state_a, state_b;
  reg         state_c, state_d, state_e;
  reg         state_w;                                     /* wake state                   */
  reg         sync_reg;                                    /* ph1 sampled sync signal      */
  reg         syndet_reg;                                  /* sync the machine state       */
  reg         t0_reg, t1_reg;                              /* bit identifiers              */
  reg   [1:0] gsa_reg, gsv_reg, unk_reg;                   /* NMEA string counters         */
  reg   [1:0] pgms_reg;                                    /* page ms bits                 */
  reg   [1:0] pwrctl_reg;                                  /* power control                */
  reg   [1:0] rx_ist_reg;                                  /* raw ibcs[9:8]                */
  reg   [1:0] sbrsel_nxt, sbrsel_reg;                      /* sbuf read claim              */
  reg   [1:0] sbwsel_nxt, sbwsel_reg;                      /* sbuf write claim             */
  reg   [1:0] strtrk_reg;                                  /* string "," counter           */
  reg   [1:0] tx_ist_reg;                                  /* raw obcs[9:8]                */
  reg   [1:0] tx_state_nxt, tx_state_reg;                  /* fifo-tx control              */
  reg   [2:0] baud_reg;                                    /* baud rate select             */
  reg   [2:0] mode2_reg;                                   /* gsa "mode2"                  */
  reg   [2:0] obdval_reg;                                  /* obd valid count              */
  reg   [2:0] rx_mode2_reg;                                /* rx mode2                     */
  reg   [2:0] talkid0_reg, talkid1_reg;                    /* talker id from str header    */
  reg   [3:0] cycle_reg;                                   /* clock cycle                  */
  reg   [3:0] relrom_inv_reg;                              /* rom relo addr (inv polarity) */
  reg   [3:0] sbwadd_nxt;                                  /* sbuf page write addr         */
  reg   [3:0] syncnt_reg;                                  /* sync detector                */
  reg   [4:0] serdiv_reg;                                  /* baud rate divider            */
  reg   [5:0] rc_reg;                                      /* rc read shifter              */
  reg   [7:0] din8_reg;                                    /* latest 8 bits of data_bus    */
  reg   [7:0] ibd_reg;                                     /* ibd read shifter             */
  reg   [7:0] rm_reg;                                      /* rm read shifter              */
  reg   [7:0] sbd_nxt;                                     /* sbd load data                */
  reg   [7:0] sbd_reg;                                     /* sbd read shifter             */
  reg   [8:0] rm_addr_reg;                                 /* reg mem address              */
  reg   [8:0] rxidl_reg;                                   /* rx idle line count           */
  reg   [8:0] rxlim_reg;                                   /* rx length limit counter      */
  reg   [9:0] iftch_reg;                                   /* instruction assembly         */
  reg   [9:6] lftch_reg;                                   /* latched instruction          */
  reg   [9:0] rx_levl_reg;                                 /* raw ibcs[22:12]              */
  reg  [10:0] tx_levl_reg;                                 /* raw obcs[21:12]              */
  reg  [10:0] sbcs_reg;                                    /* sbcs read shifter            */
  reg  [10:0] sbradd_reg;                                  /* sbuf read addr               */
  reg  [10:0] sbwadd_reg;                                  /* sbuf write addr              */
  reg  [11:0] sb1val_reg, sb0val_reg;                      /* sbuf page valid individual   */
  reg  [14:0] ibcs_reg;                                    /* ibcs read shifter            */
  reg  [15:0] obcs_reg;                                    /* obcs read shifter            */
  reg  [18:0] din_reg;                                     /* first 19 bits of data_bus    */
  reg  [27:0] gcs_reg;                                     /* gcs read shifter             */
  reg  [55:0] scr1_reg, scr2_reg;                          /* scratch registers            */

  reg  [`SS_IDX] ssrch_nxt, ssrch_reg;                     /* string search state          */

  /*****************************************************************************************/
  /* clock domain crossing                                                                 */
  /*****************************************************************************************/
  reg         cli_cdc      /* synthesis syn_preserve=1 */; /* clear inbound                */
  reg         clo_cdc      /* synthesis syn_preserve=1 */; /* clear outbound               */
  reg         gen_cdc      /* synthesis syn_preserve=1 */; /* gnss enable                  */
  reg         ibd_cdc      /* synthesis syn_preserve=1 */; /* latch inbound data           */
  reg         obd_cdc      /* synthesis syn_preserve=1 */; /* latch outbound data          */
  reg         sbc_cdc      /* synthesis syn_preserve=1 */; /* string buffer claim          */
  reg         sbr_cdc      /* synthesis syn_preserve=1 */; /* string buffer release        */

  reg   [2:0] arx_cdc_reg  /* synthesis syn_preserve=1 */; /* rx all                       */
  reg   [2:0] cli_cdc_reg  /* synthesis syn_preserve=1 */; /* clear inbound                */
  reg   [2:0] clo_cdc_reg  /* synthesis syn_preserve=1 */; /* clear outbound               */
  reg   [2:0] gen_cdc_reg  /* synthesis syn_preserve=1 */; /* gnss enable                  */
  reg   [2:0] htx_cdc_reg  /* synthesis syn_preserve=1 */; /* hold tx                      */
  reg   [2:0] ibd_cdc_reg  /* synthesis syn_preserve=1 */; /* latch inbound data           */
  reg   [2:0] ien_cdc_reg  /* synthesis syn_preserve=1 */; /* interrupt enable             */
  reg   [2:0] obd_cdc_reg  /* synthesis syn_preserve=1 */; /* latch outbound data          */
  reg   [2:0] pus_cdc_reg  /* synthesis syn_preserve=1 */; /* pwr-up state (opp polarity)  */
  reg   [2:0] sbc_cdc_reg  /* synthesis syn_preserve=1 */; /* string buffer claim          */
  reg   [2:0] sbr_cdc_reg  /* synthesis syn_preserve=1 */; /* string buffer release        */
  reg   [2:0] t0_cdc_reg   /* synthesis syn_preserve=1 */; /* t0                           */

  /*****************************************************************************************/
  /* debug-specific                                                                        */
  /*****************************************************************************************/
  wire  [8:0] debug_out;                                   /* debug output signals         */

  assign debug_out = (debug_ctl == 3'b111) ? 9'h0 :
                     (debug_ctl == 3'b110) ? {rx_full, rx_data} :
                     (debug_ctl == 3'b101) ? {rxidl_reg} :
                     (debug_ctl == 3'b100) ? {rxlim_reg} :
                     (debug_ctl == 3'b011) ? {1'b0, sbwsel_reg, ld_sbwa, sbw_wr,
                                              sbwadd_reg[10:7]} :
                     (debug_ctl == 3'b010) ? {2'b00, clr_ncnt, clr_nmea, nmea_end,
                                              str_comma, ld_strtrk, strtrk_reg} :
                     (debug_ctl == 3'b001) ? {1'b0, ld_sbrsel, ld_sbwsel, sblast_reg,
                                              sbwclm_reg, sbrsel_reg, sbwsel_reg} :
                                             {3'b000, ser_pls, serdiv_reg};

  /*****************************************************************************************/
  /* isa bus output                                                                    ph2 */
  /*****************************************************************************************/
  assign isa_drv  = pwob && pus_inv_reg && srvc_req;
  assign isa_out  = pwob && pus_inv_reg && srvc_req;

  /*****************************************************************************************/
  /* data bus output                                                                   ph2 */
  /*****************************************************************************************/
  assign data_drv = (enabl_reg || enram_reg) && cregn_reg && !ph2;
  assign data_out = sbcs_reg[0] || sbd_reg[0] || gcs_reg[0] || rc_reg[0] || ibcs_reg[0] ||
                    ibd_reg[0] || obcs_reg[0] || rm_reg[0] ||
                    (rdscr1_inst && scr1_reg[0]) || (rdscr2_inst && scr2_reg[0]);

  /*****************************************************************************************/
  /* service request                                                                       */
  /*****************************************************************************************/
  assign srvc_drv = state_0 && enabl_reg;
  assign srvc_req = attn_reg;

  assign fi_drv   =  !pwob && pus_inv_reg &&
                     ((state_b && (srvc_req || cycle_reg[3])) || srvc_drv);
  assign fi_out   =  !pwob && pus_inv_reg && ((state_b && cycle_reg[3]) ||
                     (srvc_drv && (!srvc_req || cycle_reg[3])));

  /*****************************************************************************************/
  /* power up status bit                                                               ph2 */
  /*****************************************************************************************/
  always @ (posedge ph2 or posedge pwr_upb) begin
    if      (pwr_upb) pus_inv_reg <= 1'b0;
    else if (ld_inst) pus_inv_reg <= wrgcs_inst || pus_inv_reg;
    end

  /*****************************************************************************************/
  /* sample bus signals                                                                ph1 */
  /*****************************************************************************************/
  always @ (negedge ph1 or posedge pwr_upb) begin
    if (pwr_upb) begin
      data_reg <= 1'b0;
      isa_reg  <= 1'b0;
      sync_reg <= 1'b0;
      end
    else begin
      data_reg <= data_bus;
      isa_reg  <= isa_bus;
      sync_reg <= sync;
      end
    end

  /*****************************************************************************************/
  /* bus synchronization                                                               ph2 */
  /*****************************************************************************************/
  always @ (posedge ph2 or posedge pwob) begin
    if (pwob) begin
      syncnt_reg <= 4'b0;
      syndet_reg <= 1'b0;
      end
    else begin
      syncnt_reg <= (sync_reg) ? (syncnt_reg + 1'b1) : 4'b0000;
      syndet_reg <= !sync_reg && ~|(syncnt_reg ^ 4'b1010) && powoff_inv_reg;
      end
    end

  /*****************************************************************************************/
  /* bus state machine                                                                 ph2 */
  /*****************************************************************************************/
  /*                                                                                       */
  /*   state_x         eeeecccc88880000111122225555aaaa4444999933336666ddddbbbb            */
  /*   cycle_reg       12481248124812481248124812481248124812481248124812481248            */
  /*   ld_cycle        ...1...1...1...1...1...1...1...1...1...1...1...1...1...1            */
  /*   ld_inst         .......................................................1            */
  /*   address         ..............1111111111111111..........................            */
  /*   sync            ............................................1111111111..            */
  /*                                                                                       */
  /*   state_ctl       1111111111111111111.....................................            */
  /*   state_byte      ....1111....1111....1111....1111....1111....1111....1111            */
  /*                                                                                       */
  /*   sbcs_reg        .......1ddddddddddd.....................................            */
  /*   sbd_reg        1.......1.......1.......1.......1.......1.......1.......1            */
  /*   gcs_reg         ...1dddddddddddddddd....................................            */
  /*   rc_reg          .....1dddddd............................................            */
  /*   ibcs_reg        ......1dddddddddddddddd.................................            */
  /*   ibd_reg         .......1...1.......1.......1.......1....................            */
  /*   obcs_reg        ......1ddddddddddddddd..................................            */
  /*                                                                                       */
  /*   cli_cdc         ........................................................1           */
  /*   clo_cdc         ........................................................1           */
  /*   ibst_cdc        1.......................................................            */
  /*   ibd_cdc         ........1.......1.......1.......1.......1...............            */
  /*   obst_cdc        1.......................................................            */
  /*   obd_cdc         ............1.......1.......1.......1.......1...........            */
  /*   rxst_cdc        1.......................................................            */
  /*   sbc_cdc         ..........1.............................................            */
  /*   sbr_cdc         ........................................................1           */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge ph2 or posedge pwr_upb) begin
    if (pwr_upb) cycle_reg <= 4'b0000;
    else         cycle_reg <= {( !cycle_reg[3]   && cycle_reg[2] && !syndet_reg),
                               (~|cycle_reg[3:2] && cycle_reg[1] && !syndet_reg),
                               (~|cycle_reg[3:1] && !syndet_reg),
                               (cycle_reg[3] || syndet_reg)};
    end

  always @ (posedge ph2 or posedge pwr_upb) begin
    if (pwr_upb) begin
      state_w <= 1'b0;
      state_e <= 1'b0;
      state_c <= 1'b0;
      state_8 <= 1'b0;
      state_0 <= 1'b0;
      state_1 <= 1'b0;
      state_2 <= 1'b0;
      state_5 <= 1'b0;
      state_a <= 1'b0;
      state_4 <= 1'b0;
      state_9 <= 1'b0;
      state_3 <= 1'b0;
      state_6 <= 1'b0;
      state_d <= 1'b0;
      state_b <= 1'b0;
      end
    else if (ld_cycle) begin
      state_w <= 1'b1;
      state_e <= state_b ||  syndet_reg || !state_w;
      state_c <= state_e && !syndet_reg;
      state_8 <= state_c && !syndet_reg;
      state_0 <= state_8 && !syndet_reg;
      state_1 <= state_0 && !syndet_reg;
      state_2 <= state_1 && !syndet_reg;
      state_5 <= state_2 && !syndet_reg;
      state_a <= state_5 && !syndet_reg;
      state_4 <= state_a && !syndet_reg;
      state_9 <= state_4 && !syndet_reg;
      state_3 <= state_9 && !syndet_reg;
      state_6 <= state_3 && !syndet_reg;
      state_d <= state_6 && !syndet_reg;
      state_b <= state_d && !syndet_reg;
      end
    end

  always @ (posedge ph2 or posedge pwr_upb) begin
    if (pwr_upb) begin
      t0_reg <= 1'b0;
      t1_reg <= 1'b0;
      end
    else begin
      t0_reg <= state_b && cycle_reg[3];
      t1_reg <= state_e && cycle_reg[0];
      end
    end

  assign ld_cycle   = cycle_reg[3];
  assign ld_inst    = state_b && cycle_reg[3];
  assign state_ctl  = state_e || state_c || state_8 || state_0 ||
                      (state_1 && |cycle_reg[2:0]);
  assign state_byte = state_c || state_0 || state_2 || state_a ||
                      state_9 || state_6 || state_b;

  /*****************************************************************************************/
  /* data bus input                                                                    ph2 */
  /*****************************************************************************************/
  always @ (posedge ph2 or posedge pwob) begin
    if (pwob) begin
      din8_reg <= 8'h0;
      din_reg  <= 19'h0;
      end
    else begin
      din8_reg <= {data_reg, din8_reg[7:1]};
      if (state_ctl) din_reg <= {data_reg, din_reg[18:1]};
      end
    end

  /*****************************************************************************************/
  /* instruction register and decodes                                                  ph2 */
  /*****************************************************************************************/
  always @ (posedge ph2 or posedge pwob) begin
    if          (pwob) iftch_reg <= 10'h000;
    else if (sync_reg) iftch_reg <= {isa_reg, iftch_reg[9:1]};
    end

  always @ (posedge ph2 or posedge pwob) begin
    if (pwob) begin
      cregn_reg      <= 1'b0;
      daddc_reg      <= 1'b0;
      lftch_reg      <= 4'h0;
      pfadc_reg      <= 1'b0;
      powoff_inv_reg <= 1'b0;
      regnc_reg      <= 1'b0;
      end
    else if (ld_inst) begin
      cregn_reg      <= ~|(iftch_reg[5:0] ^ 6'b111000);
      daddc_reg      <= ~|(iftch_reg ^ 10'h270);
      lftch_reg      <=    iftch_reg[9:6];
      pfadc_reg      <= ~|(iftch_reg ^ 10'h3f0);
      powoff_inv_reg <=  |(iftch_reg ^ 10'h060);
      regnc_reg      <= ~|(iftch_reg[5:0] ^ 6'b101000);
      end
    end

  /*****************************************************************************************/
  /* chip enable decodes                                                               ph2 */
  /*****************************************************************************************/
  always @ (posedge ph2 or posedge pwob) begin
    if (pwob) begin
      enabl_reg <= 1'b0;
      enram_reg <= 1'b0;
      end
    else if (ld_inst && (daddc_reg || pfadc_reg)) begin
      enabl_reg <= pfadc_reg && ~|(din_reg[7:0] ^ 8'hf1);
      enram_reg <= pfadc_reg && ~|(din_reg[7:0] ^ 8'hf2);
      end
    end

  /*****************************************************************************************/
  /* peripheral register decodes                                                       ph2 */
  /*****************************************************************************************/
  assign rdsbcs_inst = enabl_reg && cregn_reg && ~|(lftch_reg[9:6] ^ 4'b0000);
  assign rdsbd_inst  = enabl_reg && cregn_reg && ~|(lftch_reg[9:6] ^ 4'b0001);
  assign rdgcs_inst  = enabl_reg && cregn_reg && ~|(lftch_reg[9:6] ^ 4'b0100);
  assign rdrc_inst   = enabl_reg && cregn_reg && ~|(lftch_reg[9:6] ^ 4'b0101);
  assign rdscr1_inst = enabl_reg && cregn_reg && ~|(lftch_reg[9:6] ^ 4'b0110);
  assign rdscr2_inst = enabl_reg && cregn_reg && ~|(lftch_reg[9:6] ^ 4'b0111);
  assign rdibcs_inst = enabl_reg && cregn_reg && ~|(lftch_reg[9:6] ^ 4'b1000);
  assign rdibd_inst  = enabl_reg && cregn_reg && ~|(lftch_reg[9:6] ^ 4'b1001);
  assign rdobcs_inst = enabl_reg && cregn_reg && ~|(lftch_reg[9:6] ^ 4'b1100);
  assign rdobd_inst  = enabl_reg && cregn_reg && ~|(lftch_reg[9:6] ^ 4'b1101);

  assign wrsbcs_inst = enabl_reg && regnc_reg && ~|(lftch_reg[9:6] ^ 4'b0000);
  assign wrsbc_inst  = enabl_reg && regnc_reg && ~|(lftch_reg[9:6] ^ 4'b0010);
  assign wrsbr_inst  = enabl_reg && regnc_reg && ~|(lftch_reg[9:6] ^ 4'b0011);
  assign wrgcs_inst  = enabl_reg && regnc_reg && ~|(lftch_reg[9:6] ^ 4'b0100);
  assign wrrc_inst   = enabl_reg && regnc_reg && ~|(lftch_reg[9:6] ^ 4'b0101);
  assign wrscr1_inst = enabl_reg && regnc_reg && ~|(lftch_reg[9:6] ^ 4'b0110);
  assign wrscr2_inst = enabl_reg && regnc_reg && ~|(lftch_reg[9:6] ^ 4'b0111);
  assign wribcs_inst = enabl_reg && regnc_reg && ~|(lftch_reg[9:6] ^ 4'b1000);
  assign wrobcs_inst = enabl_reg && regnc_reg && ~|(lftch_reg[9:6] ^ 4'b1100);
  assign wrobd_inst  = enabl_reg && regnc_reg && ~|(lftch_reg[9:6] ^ 4'b1101);

  /*****************************************************************************************/
  /* string buffer control/status                                                      ph2 */
  /*****************************************************************************************/
  always @ (posedge ph2 or posedge pwob) begin
    if (pwob) begin
      sbradd_reg <= 11'h0;
      end
    else if ((ld_inst && wrsbcs_inst) || sbr_ainc) begin
      sbradd_reg <= (wrsbcs_inst) ? {din_reg[11:8], din_reg[18:12]} : sbradd_reg + 1'b1;
      end
    end

  assign ld_sbcs = rdsbcs_inst && state_c && cycle_reg[3];

  always @ (posedge ph2 or posedge pwob) begin
    if (pwob) sbcs_reg  <= 11'h0;
    else      sbcs_reg  <= (ld_sbcs) ? {sbradd_reg[6:0], sbradd_reg[10:7]} :
                                       {1'b0, sbcs_reg[10:1]};
    end

  /*****************************************************************************************/
  /* string buffer read side                                                           ph2 */
  /*****************************************************************************************/
  assign sbr_prd   = ~|(iftch_reg[5:0] ^ 6'b111000) && state_b && cycle_reg[2];
  assign sbr_pld   = ~|(iftch_reg[5:0] ^ 6'b111000) && state_b && cycle_reg[3];
  assign sbr_ainc  =  rdsbd_inst && state_byte && cycle_reg[1];
  assign sbr_rd    = (rdsbd_inst && state_byte && cycle_reg[2]) || sbr_prd;
  assign sbr_ld    = (rdsbd_inst && state_byte && cycle_reg[3]) || sbr_pld;

  assign sbradd_mux = (wrsbcs_inst) ? {din_reg[11:8], din_reg[18:12]} : sbradd_reg;

  assign sb00_rd = sbrsel_reg[0] && (sbradd_mux[10:9] == 2'b00) && sbr_rd;
  assign sb01_rd = sbrsel_reg[0] && (sbradd_mux[10:9] == 2'b01) && sbr_rd;
  assign sb02_rd = sbrsel_reg[0] && (sbradd_mux[10:9] == 2'b10) && sbr_rd;
  assign sb10_rd = sbrsel_reg[1] && (sbradd_mux[10:9] == 2'b00) && sbr_rd;
  assign sb11_rd = sbrsel_reg[1] && (sbradd_mux[10:9] == 2'b01) && sbr_rd;
  assign sb12_rd = sbrsel_reg[1] && (sbradd_mux[10:9] == 2'b10) && sbr_rd;

  always @ (sbradd_mux or sbrsel_reg or sb00_dout or sb01_dout or sb02_dout or sb10_dout or
            sb11_dout or sb12_dout) begin
    case ({sbrsel_reg, sbradd_mux[10:9]})
      4'b0100: sbd_nxt = sb00_dout;
      4'b0101: sbd_nxt = sb01_dout;
      4'b0110: sbd_nxt = sb02_dout;
      4'b1000: sbd_nxt = sb10_dout;
      4'b1001: sbd_nxt = sb11_dout;
      4'b1010: sbd_nxt = sb12_dout;
      default: sbd_nxt = 8'h0;
      endcase
    end

  always @ (posedge ph2 or posedge pwob) begin
    if (pwob) sbd_reg <= 8'h0;
    else      sbd_reg <= (sbr_ld) ? sbd_nxt : {1'b0, sbd_reg[7:1]};
    end

  /*****************************************************************************************/
  /* string buffer                                                                 ph2/clk */
  /*****************************************************************************************/
`ifdef SIM_VERSION

  ram_512x8    SB12_RAM ( .RADDR(sbradd_mux[8:0]), .RCLK(ph2), .RCLKE(sb12_rd),
                          .RE(sb12_rd), .RDATA(sb12_dout), .WADDR(sbwadd_reg[8:0]),
                          .WCLK(clk), .WCLKE(sb12_wr), .WE(sb12_wr), .WDATA(rx_data) );
  ram_512x8    SB11_RAM ( .RADDR(sbradd_mux[8:0]), .RCLK(ph2), .RCLKE(sb11_rd),
                          .RE(sb11_rd), .RDATA(sb11_dout), .WADDR(sbwadd_reg[8:0]),
                          .WCLK(clk), .WCLKE(sb11_wr), .WE(sb11_wr), .WDATA(rx_data) );
  ram_512x8    SB10_RAM ( .RADDR(sbradd_mux[8:0]), .RCLK(ph2), .RCLKE(sb10_rd),
                          .RE(sb10_rd), .RDATA(sb10_dout), .WADDR(sbwadd_reg[8:0]),
                          .WCLK(clk), .WCLKE(sb10_wr), .WE(sb10_wr), .WDATA(rx_data) );
  ram_512x8    SB02_RAM ( .RADDR(sbradd_mux[8:0]), .RCLK(ph2), .RCLKE(sb02_rd),
                          .RE(sb02_rd), .RDATA(sb02_dout), .WADDR(sbwadd_reg[8:0]),
                          .WCLK(clk), .WCLKE(sb02_wr), .WE(sb02_wr), .WDATA(rx_data) );
  ram_512x8    SB01_RAM ( .RADDR(sbradd_mux[8:0]), .RCLK(ph2), .RCLKE(sb01_rd),
                          .RE(sb01_rd), .RDATA(sb01_dout), .WADDR(sbwadd_reg[8:0]),
                          .WCLK(clk), .WCLKE(sb01_wr), .WE(sb01_wr), .WDATA(rx_data) );
  ram_512x8    SB00_RAM ( .RADDR(sbradd_mux[8:0]), .RCLK(ph2), .RCLKE(sb00_rd),
                          .RE(sb00_rd), .RDATA(sb00_dout), .WADDR(sbwadd_reg[8:0]),
                          .WCLK(clk), .WCLKE(sb00_wr), .WE(sb00_wr), .WDATA(rx_data) );
`else

  SB_RAM512x8  SB12_RAM ( .RADDR(sbradd_mux[8:0]), .RCLK(ph2), .RCLKE(sb12_rd),
                          .RE(sb12_rd), .RDATA(sb12_dout), .WADDR(sbwadd_reg[8:0]),
                          .WCLK(clk), .WCLKE(sb12_wr), .WE(sb12_wr), .WDATA(rx_data) );
  SB_RAM512x8  SB11_RAM ( .RADDR(sbradd_mux[8:0]), .RCLK(ph2), .RCLKE(sb11_rd),
                          .RE(sb11_rd), .RDATA(sb11_dout), .WADDR(sbwadd_reg[8:0]),
                          .WCLK(clk), .WCLKE(sb11_wr), .WE(sb11_wr), .WDATA(rx_data) );
  SB_RAM512x8  SB10_RAM ( .RADDR(sbradd_mux[8:0]), .RCLK(ph2), .RCLKE(sb10_rd),
                          .RE(sb10_rd), .RDATA(sb10_dout), .WADDR(sbwadd_reg[8:0]),
                          .WCLK(clk), .WCLKE(sb10_wr), .WE(sb10_wr), .WDATA(rx_data) );
  SB_RAM512x8  SB02_RAM ( .RADDR(sbradd_mux[8:0]), .RCLK(ph2), .RCLKE(sb02_rd),
                          .RE(sb02_rd), .RDATA(sb02_dout), .WADDR(sbwadd_reg[8:0]),
                          .WCLK(clk), .WCLKE(sb02_wr), .WE(sb02_wr), .WDATA(rx_data) );
  SB_RAM512x8  SB01_RAM ( .RADDR(sbradd_mux[8:0]), .RCLK(ph2), .RCLKE(sb01_rd),
                          .RE(sb01_rd), .RDATA(sb01_dout), .WADDR(sbwadd_reg[8:0]),
                          .WCLK(clk), .WCLKE(sb01_wr), .WE(sb01_wr), .WDATA(rx_data) );
  SB_RAM512x8  SB00_RAM ( .RADDR(sbradd_mux[8:0]), .RCLK(ph2), .RCLKE(sb00_rd),
                          .RE(sb00_rd), .RDATA(sb00_dout), .WADDR(sbwadd_reg[8:0]),
                          .WCLK(clk), .WCLKE(sb00_wr), .WE(sb00_wr), .WDATA(rx_data) );
`endif

  /*****************************************************************************************/
  /* gnss control/status register                                                      ph2 */
  /*****************************************************************************************/
  always @ (posedge ph2 or posedge pwr_upb) begin
    if (pwr_upb) begin
      baud_reg   <=  3'b0;
      inten_reg  <=  1'b0;
      rxen_reg   <=  1'b0;
      pwrctl_reg <=  2'b0;
      end
    else if (ld_inst && wrgcs_inst) begin
      baud_reg   <=  din_reg[7:5];
      inten_reg  <=  din_reg[4];
      rxen_reg   <= (din_reg[13] && !din_reg[12]) || (~&din_reg[13:12] && rxen_reg);
      pwrctl_reg <=  din_reg[15:14];
      end
    end

  assign gnss_en  = pwrctl_reg[0] || (~|pwrctl_reg && rxen_reg);
  assign osc_en   = pwrctl_reg[0] || !pwob;

  assign ld_gcs   = rdgcs_inst && state_e && cycle_reg[3];

  always @ (posedge ph2 or posedge pwob) begin
    if (pwob) gcs_reg  <= 28'h0;
    else      gcs_reg  <= (ld_gcs) ? {sbr_valid, 1'b0, talk_id, pwrctl_reg, 3'b000, 
                                      rx_mode2_reg, baud_reg, inten_reg} :
                                     {1'b0, gcs_reg[27:1]};
    end

  /*****************************************************************************************/
  /* rom control register                                                              ph2 */
  /*****************************************************************************************/
  always @ (posedge ph2 or posedge pwr_upb) begin
    if (pwr_upb) begin
      disrom_reg     <= 1'b0;
      relrom_inv_reg <= 4'h0;
      rwrom_reg      <= 1'b0;
      end
    else if (ld_inst && wrrc_inst) begin
      disrom_reg     <=  din_reg[7];
      relrom_inv_reg <= ~din_reg[11:8];
      rwrom_reg      <= !din_reg[7] && din_reg[6];
      end
    end

  assign ld_rc = rdrc_inst && state_c && cycle_reg[1];

  always @ (posedge ph2 or posedge pwob) begin
    if (pwob) rc_reg <= 6'h0;
    else      rc_reg <= (ld_rc) ? {~relrom_inv_reg, disrom_reg, rwrom_reg} :
                                  {1'b0, rc_reg[5:1]};
    end

  /*****************************************************************************************/
  /* rom (instruction memory) interface                                                ph2 */
  /*****************************************************************************************/
  always @ (posedge ph2 or posedge pwr_upb) begin
    if (pwr_upb) begin
      pgms_reg      <= 2'b0;
      enrom_inv_reg <= 1'b0;
      end
    else begin
      if (state_a && |cycle_reg[1:0]) pgms_reg      <= {isa_reg, pgms_reg[1]};
      if (ld_inst && ~|pgms_reg)      enrom_inv_reg <= disrom_reg;
      end
    end

  assign enabl_rd  = !enrom_inv_reg;
  assign enabl_wr  = rwrom_reg;
  assign page_addr = ~relrom_inv_reg;

  /*****************************************************************************************/
  /* scratch1 register                                                                 ph2 */
  /*****************************************************************************************/
  assign en_scr1  =  rdscr1_inst || wrscr1_inst;
  assign scr1_nxt = (wrscr1_inst) ? data_reg : scr1_reg[0];

  always @ (posedge ph2 or posedge pwr_upb) begin
    if      (pwr_upb) scr1_reg <= 56'h0;
    else if (en_scr1) scr1_reg <= {scr1_nxt, scr1_reg[55:1]};
    end

  /*****************************************************************************************/
  /* scratch2 register                                                                 ph2 */
  /*****************************************************************************************/
  assign en_scr2  =  rdscr2_inst || wrscr2_inst;
  assign scr2_nxt = (wrscr2_inst) ? data_reg : scr2_reg[0];

  always @ (posedge ph2 or posedge pwr_upb) begin
    if      (pwr_upb) scr2_reg <= 56'h0;
    else if (en_scr2) scr2_reg <= {scr2_nxt, scr2_reg[55:1]};
    end

  /*****************************************************************************************/
  /* inbound buffer control/status                                                     ph2 */
  /*****************************************************************************************/
  always @ (posedge ph2 or posedge pwr_upb) begin
    if                     (pwr_upb) iball_reg <= 1'b0;
    else if (ld_inst && wribcs_inst) iball_reg <= din_reg[7];
    end

  assign ld_ibcs = rdibcs_inst && state_c && cycle_reg[2];

  always @ (posedge ph2 or posedge pwob) begin
    if (pwob) ibcs_reg <= 15'h0;
    else      ibcs_reg <= (ld_ibcs) ? {rx_levl_reg, rx_ist_reg[1], 2'b00, rx_ist_reg[0],
                                       iball_reg} :
                                      {1'b0, ibcs_reg[14:1]};
    end

  /*****************************************************************************************/
  /* inbound buffer data                                                               ph2 */
  /*****************************************************************************************/
  assign ibd_en = (state_8 && rx_1byte) || (state_1 && rx_2byte) ||
                  (state_5 && rx_3byte) || (state_4 && rx_4byte);

  assign ld_ibd  = rdibd_inst && ((state_c && cycle_reg[3]) || (ibd_en && cycle_reg[3]));
  assign ibd_nxt = (state_c) ? {5'h00, rx_count} : rx_fifo_out;

  always @ (posedge ph2 or posedge pwob) begin
    if (pwob) ibd_reg <= 8'b0;
    else      ibd_reg <= (ld_ibd) ? ibd_nxt : {1'b0, ibd_reg[7:1]};
    end

  /*****************************************************************************************/
  /* outbound buffer control/status                                                    ph2 */
  /*****************************************************************************************/
  always @ (posedge ph2 or posedge pwob) begin
    if                        (pwob) hldtx_reg <= 1'b0;
    else if (ld_inst && wrobcs_inst) hldtx_reg <= din_reg[7];
    end

  assign ld_obcs = rdobcs_inst && state_c && cycle_reg[2];

  always @ (posedge ph2 or posedge pwob) begin
    if (pwob) obcs_reg <= 16'h0;
    else      obcs_reg <= (ld_obcs) ? {tx_levl_reg, tx_ist_reg[1], 2'b00, tx_ist_reg[0],
                                       hldtx_reg} :
                                      {1'b0, obcs_reg[15:1]};
    end

  /*****************************************************************************************/
  /* outbound buffer data                                                              ph2 */
  /*****************************************************************************************/
  always @ (posedge ph2 or posedge pwob) begin
    if                                       (pwob) obdval_reg <= 3'b0;
    else if (wrobd_inst && state_0 && cycle_reg[0]) obdval_reg <= din8_reg[6:4];
    end

  assign obd_en  = ( state_1                                   && (obdval_reg == 3'b001)) ||
                   ((state_1 || state_5)                       && (obdval_reg == 3'b010)) ||
                   ((state_1 || state_5 || state_4)            && (obdval_reg == 3'b011)) ||
                   ((state_1 || state_5 || state_4 || state_3) && (obdval_reg == 3'b100));

  /*****************************************************************************************/
  /* reg memory                                                                        ph2 */
  /*****************************************************************************************/
  assign ld_rm_addr = enram_reg && (cregn_reg || regnc_reg) && t0_reg;
  assign i_rm_addr  = (enram_reg && cregn_reg && state_byte && cycle_reg[1]) || rm_wr;
  assign rm_pld     = ~|(iftch_reg[5:0] ^ 6'b111000) && state_b && cycle_reg[3];
  assign rm_prd     = ~|(iftch_reg[5:0] ^ 6'b111000) && state_b && cycle_reg[2];
  assign rm_ld      = (enram_reg && cregn_reg && state_byte && cycle_reg[3]) || rm_pld;
  assign rm_rd      = (enram_reg && cregn_reg && state_byte && cycle_reg[2]) || rm_prd;

  always @ (posedge ph2 or posedge pwob) begin
    if (pwob) rm_wr <= 1'b0;
    else      rm_wr <= enram_reg && regnc_reg && state_byte && cycle_reg[3]; 
    end

  always @ (posedge ph2 or posedge pwob) begin
    if (pwob) rm_addr_reg <= 9'h0;
    else if (ld_rm_addr || i_rm_addr) begin
      rm_addr_reg <= (ld_rm_addr) ? {2'h0, lftch_reg, 3'h0} : rm_addr_reg + 1'b1;
      end
    end

  assign rm_addr_mux = (rm_prd) ? {2'h0, iftch_reg[9:6], 3'h0} : rm_addr_reg;

`ifdef SIM_VERSION

  ram_512x8    REGM     ( .RADDR(rm_addr_mux), .RCLK(ph2), .RCLKE(rm_rd), .RE(rm_rd),
                          .RDATA(rm_nxt), .WADDR(rm_addr_reg), .WCLK(ph2),
                          .WCLKE(rm_wr), .WE(rm_wr), .WDATA(din8_reg) );
`else

  SB_RAM512x8  REGM     ( .RADDR(rm_addr_mux), .RCLK(ph2), .RCLKE(rm_rd), .RE(rm_rd),
                          .RDATA(rm_nxt), .WADDR(rm_addr_reg), .WCLK(ph2),
                          .WCLKE(rm_wr), .WE(rm_wr), .WDATA(din8_reg) );
`endif

  always @ (posedge ph2 or posedge pwob) begin
    if (pwob) rm_reg <= 8'h0;
    else      rm_reg <= (rm_ld) ? rm_nxt : {1'b0, rm_reg[7:1]};
    end

  /*****************************************************************************************/
  /* clock domain crossing                                                      ph2 -> clk */
  /*****************************************************************************************/
  always @ (posedge ph2 or posedge pwob) begin
    if (pwob) begin
      cli_cdc <= 1'b0;
      clo_cdc <= 1'b0;
      gen_cdc <= 1'b0;
      ibd_cdc <= 1'b0;
      obd_cdc <= 1'b0;
      sbc_cdc <= 1'b0;
      sbr_cdc <= 1'b0;
      end
    else begin
      cli_cdc <= ld_inst && wribcs_inst && din_reg[0];
      clo_cdc <= ld_inst && wrobcs_inst && din_reg[0];
      gen_cdc <= gnss_en;
      ibd_cdc <= rdibd_inst && ibd_en && cycle_reg[2];
      obd_cdc <= rdobd_inst && obd_en && cycle_reg[3];
      sbc_cdc <= wrsbc_inst && state_b && cycle_reg[1];
      sbr_cdc <= wrsbr_inst && ld_inst;
      end
    end

  always @ (posedge clk or posedge pwob) begin
    if (pwob) begin
      arx_cdc_reg <= 3'b0;
      cli_cdc_reg <= 3'b0;
      clo_cdc_reg <= 3'b0;
      gen_cdc_reg <= 3'b0;
      htx_cdc_reg <= 3'b0;
      ibd_cdc_reg <= 3'b0;
      ien_cdc_reg <= 3'b0;
      obd_cdc_reg <= 3'b0;
      pus_cdc_reg <= 3'b0;
      sbc_cdc_reg <= 3'b0;
      sbr_cdc_reg <= 3'b0;
      t0_cdc_reg  <= 3'b0;
      end
    else begin
      arx_cdc_reg <= {iball_reg,     arx_cdc_reg[2:1]};
      cli_cdc_reg <= {  cli_cdc,     cli_cdc_reg[2:1]};
      clo_cdc_reg <= {  clo_cdc,     clo_cdc_reg[2:1]};
      gen_cdc_reg <= {  gen_cdc,     gen_cdc_reg[2:1]};
      htx_cdc_reg <= {hldtx_reg,     htx_cdc_reg[2:1]};
      ibd_cdc_reg <= {  ibd_cdc,     ibd_cdc_reg[2:1]};
      ien_cdc_reg <= {inten_reg,     ien_cdc_reg[2:1]};
      obd_cdc_reg <= {  obd_cdc,     obd_cdc_reg[2:1]};
      pus_cdc_reg <= {  pus_inv_reg, pus_cdc_reg[2:1]};
      sbc_cdc_reg <= {  sbc_cdc,     sbc_cdc_reg[2:1]};
      sbr_cdc_reg <= {  sbr_cdc,     sbr_cdc_reg[2:1]};
      t0_cdc_reg  <= {   t0_reg,     t0_cdc_reg[2:1]};
      end
    end

  always @ (posedge clk or posedge pwr_upb) begin
    if (pwr_upb) begin
      rx_mode2_reg <=  3'b0;
      rx_ist_reg   <=  2'b0;
      rx_levl_reg  <= 10'h0;
      tx_ist_reg   <=  2'b0;
      tx_levl_reg  <= 11'h0;
      end
    else if (t0_cdc_reg[1] && !t0_cdc_reg[0]) begin
      rx_mode2_reg <= mode2_reg;
      rx_ist_reg   <= {rx_ov, !rx_empt};
      rx_levl_reg  <= rx_levl;
      tx_ist_reg   <= {tx_full, !tx_empt};
      tx_levl_reg  <= (11'h400 - tx_levl);
      end
    end

  assign rx_1byte   = |rx_levl_reg;
  assign rx_2byte   = |rx_levl_reg[9:1];
  assign rx_3byte   = |rx_levl_reg[9:2] || &rx_levl_reg[1:0];
  assign rx_4byte   = |rx_levl_reg[9:2];
  assign rx_count   = (rx_4byte) ? 3'b100 : {1'b0, rx_levl_reg[1:0]};

  assign rx_fifo_rd = ibd_cdc_reg[1] && !ibd_cdc_reg[0];
  assign rx_purge   = cli_cdc_reg[1] && !cli_cdc_reg[0];
  assign rxmode_clr =                   !gen_cdc_reg[0];
  assign tx_fifo_wr = obd_cdc_reg[1] && !obd_cdc_reg[0];
  assign tx_purge   = clo_cdc_reg[1] && !clo_cdc_reg[0];
  assign sbrd_clm   = sbc_cdc_reg[1] && !sbc_cdc_reg[0];
  assign sbrd_rel   = sbr_cdc_reg[1] && !sbr_cdc_reg[0];

  /*****************************************************************************************/
  /* clk-side interrupt                                                                clk */
  /*****************************************************************************************/
  always @ (posedge clk or posedge pwr_upb) begin
    if (pwr_upb) attn_reg <= 1'b0;
    else         attn_reg <= ien_cdc_reg && (attn_reg ||
                             (sbwr_rel && |sbwsel_reg && gsa_mode2 && rmcval_reg));
    end

  /*****************************************************************************************/
  /* serial clock                                                                      clk */
  /*****************************************************************************************/
  always @ (posedge clk or posedge pwr_upb) begin
    if (pwr_upb) begin
      serdiv_reg <= 4'h0;
      ser_pls    <= 1'b0;
      end
    else begin
      serdiv_reg <= serdiv_reg + 1'b1;
      ser_pls    <= (baud_reg == 3'b100) ? &serdiv_reg[2:0] :
                    (baud_reg == 3'b101) ? &serdiv_reg[3:0] :
                    (baud_reg == 3'b110) ? &serdiv_reg[4:0] : &serdiv_reg[1:0];
      end
    end

  /*****************************************************************************************/
  /* serial receive                                                                    clk */
  /*****************************************************************************************/
  assign rx_din_m = (baud_reg == 3'b001) ? tx_dout : rx_din;

  always @ (posedge clk or posedge pwr_upb) begin
    if (pwr_upb) begin
      rxidl_reg <= 9'h0;
      rx_idle   <= 1'b0;
      end
    else if (ser_pls) begin
      rxidl_reg <= (rx_din) ? rxidl_reg - 1'b1 : 9'h1ff;
      rx_idle   <=  rx_din && (~|rxidl_reg || rx_idle);
      end
    end

  assign rx_read  = ssrch_reg[`ID_HD1]  || ssrch_reg[`ID_HD3]  || ssrch_reg[`ID_HD5]  ||
                    ssrch_reg[`ID_SID1] || ssrch_reg[`ID_SID3] ||
                    ssrch_reg[`ID_RMC1] || ssrch_reg[`ID_RMC3] || ssrch_reg[`ID_RMC5] ||
                    ssrch_reg[`ID_VTG1] || ssrch_reg[`ID_VTG3] || ssrch_reg[`ID_VTG5] ||
                                           ssrch_reg[`ID_GGA3] || ssrch_reg[`ID_GGA5] ||
                                           ssrch_reg[`ID_GLL3] || ssrch_reg[`ID_GLL5] ||
                                                                  ssrch_reg[`ID_GSA5] ||
                                                                  ssrch_reg[`ID_GSV5] ||
                    ssrch_reg[`ID_ASM1] || ssrch_reg[`ID_ESC1] || ssrch_reg[`ID_ESC3] ||
                    ssrch_reg[`ID_RDEX];

  assign rx_fifo_wr = rx_full && ((rx_read && arx_cdc_reg[0]) ||
                      ssrch_reg[`ID_ESC1] || ssrch_reg[`ID_ESC3]) && !rx_limit;

  serial_rx    RCVR     ( .rx_ovr(), .rx_byte(rx_data), .rx_full(rx_full),
                          .rx_run(), .auto_trig(auto_trig), .cks_mode(!pus_cdc_reg[1]),
                          .clk(clk), .rx_read(rx_read), .rx_din(rx_din_m), .rx_sync(ser_pls),
                          .s_reset(!pus_cdc_reg[0]) );

  fifo_fpga    RX_FIFO  ( .data_out(rx_fifo_out), .fifo_empt(rx_empt), .fifo_full(), 
                          .fifo_ov(rx_ov), .fill_levl(rx_levl), .clk(clk), .data_in(rx_data),
                          .fifo_purge(rx_purge), .fifo_rd(rx_fifo_rd), .fifo_wr(rx_fifo_wr),
                          .reset(pwr_upb) );

  /*****************************************************************************************/
  /* serial transmit                                                                   clk */
  /*****************************************************************************************/
  always @ (tx_state_reg or tx_empty or htx_cdc_reg) begin
    casex ({tx_state_reg, tx_empty, htx_cdc_reg[0]})
      4'b0010: tx_state_nxt = 2'b01;
      4'b01xx: tx_state_nxt = 2'b11;
      4'b11xx: tx_state_nxt = 2'b10;
      default: tx_state_nxt = 2'b00;
      endcase
    end

  always @ (posedge clk or posedge pwr_upb) begin
    if (pwr_upb) tx_state_reg <= 2'b00;
    else         tx_state_reg <= tx_state_nxt;
    end

  assign tx_fifo_rd = !tx_state_reg[1] && tx_state_reg[0];
  assign tx_write   = &tx_state_reg;

  serial_tx    XMIT     ( .tx_done(), .tx_dout(tx_dout), .tx_empty(tx_empty),
                          .tx_run(), .auto_trig(auto_trig), .cks_mode(!pus_cdc_reg[0]),
                          .clk(clk), .s_reset(!pus_cdc_reg[0]), .tx_data(tx_data),
                          .tx_sync(ser_pls), .tx_write(tx_write) );

  fifo2_fpga   TX_FIFO  ( .data_out(tx_data), .fifo_empt(tx_empt), .fifo_full(tx_full),
                          .fifo_ov(), .fill_levl(tx_levl), .clk(clk), .data_in(din8_reg),
                          .fifo_purge(tx_purge), .fifo_rd(tx_fifo_rd), .fifo_wr(tx_fifo_wr),
                          .reset(pwr_upb) );

  /*****************************************************************************************/
  /* receive message length limiter                                                    clk */
  /*****************************************************************************************/
  assign ld_rxlim  =  ssrch_reg[`ID_ESC1] || ssrch_reg[`ID_HD1];
  assign rxlim_nxt = (ssrch_reg[`ID_ESC1]) ? 9'h1ff : 9'h04f;

  always @ (posedge clk or posedge pwr_upb) begin
    if (pwr_upb) begin
      rxlim_reg <= 9'h0;
      rx_limit  <= 1'b0;
      end
    else if (ld_rxlim || sbw_wr) begin
      rxlim_reg <= (ld_rxlim) ? rxlim_nxt : rxlim_reg - 1'b1;
      rx_limit  <= !ld_rxlim && (~|rxlim_reg || rx_limit);
      end
    end

  /*****************************************************************************************/
  /* NMEA string search state machine                                                  clk */
  /*****************************************************************************************/
  always @ (ssrch_reg or rx_data or rx_full or rx_idle) begin
    casex (ssrch_reg) //synthesis parallel_case
      `DD_IDLE: begin                                      /* (rel), clr nmea              */
        casex ({rx_full, rx_data})
          9'h0xx:  ssrch_nxt = `SS_IDLE;
          9'h124:  ssrch_nxt = `SS_HD1;                    /* $                            */
          9'h1B5:  ssrch_nxt = `SS_ESC1;                   /* UBX message                  */
          default: ssrch_nxt = `SS_RDID;                   /* discard everything else      */
          endcase
        end
      `DD_RDID:    ssrch_nxt = `SS_IDLE;                   /* rd                           */
      `DD_WAIT: begin
        casex ({rx_idle, rx_full, rx_data})
          10'h0xx: ssrch_nxt = `SS_WAIT;
          10'h124: ssrch_nxt = `SS_HD1;                    /* $                            */
          10'h1B5: ssrch_nxt = `SS_ESC1;                   /* UBX message                  */
          10'h2xx: ssrch_nxt = `SS_EXIT;
          10'h3xx: ssrch_nxt = `SS_RDEX;
          default: ssrch_nxt = `SS_RDWT;                   /* something else               */
          endcase
        end
      `DD_RDWT:    ssrch_nxt = `SS_WAIT;                   /* rd                           */
      `DD_HD1:     ssrch_nxt = `SS_HD2;                    /* rd, bcnt=80                  */
      `DD_HD2: begin
        casex ({rx_idle, rx_full, rx_data})
          10'h0xx: ssrch_nxt = `SS_HD2;
          10'h10A: ssrch_nxt = `SS_RDWT;                   /* <LF>                         */
          10'h124: ssrch_nxt = `SS_HD1;                    /* $                            */
          10'h147: ssrch_nxt = `SS_HD3;                    /* G                            */
          10'h2xx: ssrch_nxt = `SS_EXIT;
          10'h3xx: ssrch_nxt = `SS_RDEX;
          default: ssrch_nxt = `SS_UNK1;                   /* something else               */
          endcase
        end
      `DD_HD3:     ssrch_nxt = `SS_HD4;                    /* rd                           */
      `DD_HD4: begin
        casex ({rx_idle, rx_full, rx_data})
          10'h0xx: ssrch_nxt = `SS_HD4;
          10'h10A: ssrch_nxt = `SS_RDWT;                   /* <LF>                         */
          10'h124: ssrch_nxt = `SS_HD1;                    /* $                            */
          10'h14C: ssrch_nxt = `SS_HD5;                    /* L (GLONASS)                  */
          10'h14E: ssrch_nxt = `SS_HD5;                    /* N (other)                    */
          10'h150: ssrch_nxt = `SS_HD5;                    /* P (GPS)                      */
          10'h2xx: ssrch_nxt = `SS_EXIT;
          10'h3xx: ssrch_nxt = `SS_RDEX;
          default: ssrch_nxt = `SS_UNK1;                   /* something else               */
          endcase
        end
      `DD_HD5:     ssrch_nxt = `SS_SID0;                   /* rd                           */
      `DD_SID0: begin
        casex ({rx_idle, rx_full, rx_data})
          10'h0xx: ssrch_nxt = `SS_SID0;
          10'h10A: ssrch_nxt = `SS_RDWT;                   /* <LF>                         */
          10'h124: ssrch_nxt = `SS_HD1;                    /* $                            */
          10'h152: ssrch_nxt = `SS_RMC1;                   /* R (Rxx)                      */
          10'h156: ssrch_nxt = `SS_VTG1;                   /* V (Vxx)                      */
          10'h147: ssrch_nxt = `SS_SID1;                   /* G (Gxx)                      */
          10'h2xx: ssrch_nxt = `SS_EXIT;
          10'h3xx: ssrch_nxt = `SS_RDEX;
          default: ssrch_nxt = `SS_UNK1;                   /* something else               */
          endcase
        end
      `DD_SID1:    ssrch_nxt = `SS_SID2;                   /* rd                           */
      `DD_SID2: begin
        casex ({rx_idle, rx_full, rx_data})
          10'h0xx: ssrch_nxt = `SS_SID2;
          10'h10A: ssrch_nxt = `SS_RDWT;                   /* <LF>                         */
          10'h124: ssrch_nxt = `SS_HD1;                    /* $                            */
          10'h147: ssrch_nxt = `SS_GGA3;                   /* G (GGx)                      */
          10'h14C: ssrch_nxt = `SS_GLL3;                   /* L (GLx)                      */
          10'h153: ssrch_nxt = `SS_SID3;                   /* S (GSx)                      */
          10'h2xx: ssrch_nxt = `SS_EXIT;
          10'h3xx: ssrch_nxt = `SS_RDEX;
          default: ssrch_nxt = `SS_UNK1;                   /* something else               */
          endcase
        end
      `DD_SID3:    ssrch_nxt = `SS_SID4;                   /* rd                           */
      `DD_SID4: begin
        casex ({rx_idle, rx_full, rx_data})
          10'h0xx: ssrch_nxt = `SS_SID4;
          10'h10A: ssrch_nxt = `SS_RDWT;                   /* <LF>                         */
          10'h124: ssrch_nxt = `SS_HD1;                    /* $                            */
          10'h141: ssrch_nxt = `SS_GSA5;                   /* A (GSA)                      */
          10'h156: ssrch_nxt = `SS_GSV5;                   /* V (GSV)                      */
          10'h2xx: ssrch_nxt = `SS_EXIT;
          10'h3xx: ssrch_nxt = `SS_RDEX;
          default: ssrch_nxt = `SS_UNK1;                   /* something else               */
          endcase
        end
      `DD_RMC1:    ssrch_nxt = `SS_RMC2;                   /* rd                           */
      `DD_RMC2: begin
        casex ({rx_idle, rx_full, rx_data})
          10'h0xx: ssrch_nxt = `SS_RMC2;
          10'h10A: ssrch_nxt = `SS_RDWT;                   /* <LF>                         */
          10'h124: ssrch_nxt = `SS_HD1;                    /* $                            */
          10'h14D: ssrch_nxt = `SS_RMC3;                   /* M (RMx)                      */
          10'h2xx: ssrch_nxt = `SS_EXIT;
          10'h3xx: ssrch_nxt = `SS_RDEX;
          default: ssrch_nxt = `SS_UNK1;                   /* something else               */
          endcase
        end
      `DD_RMC3:    ssrch_nxt = `SS_RMC4;                   /* rd                           */
      `DD_RMC4: begin
        casex ({rx_idle, rx_full, rx_data})
          10'h0xx: ssrch_nxt = `SS_RMC4;
          10'h10A: ssrch_nxt = `SS_RDWT;                   /* <LF>                         */
          10'h124: ssrch_nxt = `SS_HD1;                    /* $                            */
          10'h143: ssrch_nxt = `SS_RMC5;                   /* C (RMC)                      */
          10'h2xx: ssrch_nxt = `SS_EXIT;
          10'h3xx: ssrch_nxt = `SS_RDEX;
          default: ssrch_nxt = `SS_UNK1;                   /* something else               */
          endcase
        end
      `DD_RMC5:    ssrch_nxt = `SS_ASM2;                   /* rd, claim, lc=0, addr        */
      `DD_VTG1:    ssrch_nxt = `SS_VTG2;                   /* rd                           */
      `DD_VTG2: begin
        casex ({rx_idle, rx_full, rx_data})
          10'h0xx: ssrch_nxt = `SS_VTG2;
          10'h10A: ssrch_nxt = `SS_RDWT;                   /* <LF>                         */
          10'h124: ssrch_nxt = `SS_HD1;                    /* $                            */
          10'h154: ssrch_nxt = `SS_VTG3;                   /* T (VTx)                      */
          10'h2xx: ssrch_nxt = `SS_EXIT;
          10'h3xx: ssrch_nxt = `SS_RDEX;
          default: ssrch_nxt = `SS_UNK1;                   /* something else               */
          endcase
        end
      `DD_VTG3:    ssrch_nxt = `SS_VTG4;                   /* rd                           */
      `DD_VTG4: begin
        casex ({rx_idle, rx_full, rx_data})
          10'h0xx: ssrch_nxt = `SS_VTG4;
          10'h10A: ssrch_nxt = `SS_RDWT;                   /* <LF>                         */
          10'h124: ssrch_nxt = `SS_HD1;                    /* $                            */
          10'h147: ssrch_nxt = `SS_VTG5;                   /* G (VTG)                      */
          10'h2xx: ssrch_nxt = `SS_EXIT;
          10'h3xx: ssrch_nxt = `SS_RDEX;
          default: ssrch_nxt = `SS_UNK1;                   /* something else               */
          endcase
        end
      `DD_VTG5:    ssrch_nxt = `SS_ASM2;                   /* rd, addr                     */
      `DD_GGA3:    ssrch_nxt = `SS_GGA4;                   /* rd                           */
      `DD_GGA4: begin
        casex ({rx_idle, rx_full, rx_data})
          10'h0xx: ssrch_nxt = `SS_GGA4;
          10'h10A: ssrch_nxt = `SS_RDWT;                   /* <LF>                         */
          10'h124: ssrch_nxt = `SS_HD1;                    /* $                            */
          10'h141: ssrch_nxt = `SS_GGA5;                   /* A (GGA)                      */
          10'h2xx: ssrch_nxt = `SS_EXIT;
          10'h3xx: ssrch_nxt = `SS_RDEX;
          default: ssrch_nxt = `SS_UNK1;                   /* something else               */
          endcase
        end
      `DD_GGA5:    ssrch_nxt = `SS_ASM2;                   /* rd, addr                     */
      `DD_GLL3:    ssrch_nxt = `SS_GLL4;                   /* rd                           */
      `DD_GLL4: begin
        casex ({rx_idle, rx_full, rx_data})
          10'h0xx: ssrch_nxt = `SS_GLL4;
          10'h10A: ssrch_nxt = `SS_RDWT;                   /* <LF>                         */
          10'h124: ssrch_nxt = `SS_HD1;                    /* $                            */
          10'h14C: ssrch_nxt = `SS_GLL5;                   /* L (GLL)                      */
          10'h2xx: ssrch_nxt = `SS_EXIT;
          10'h3xx: ssrch_nxt = `SS_RDEX;
          default: ssrch_nxt = `SS_UNK1;                   /* something else               */
          endcase
        end
      `DD_GLL5:    ssrch_nxt = `SS_ASM2;                   /* rd, addr                     */
      `DD_GSA5:    ssrch_nxt = `SS_ASM2;                   /* rd, addr, set gsa            */
      `DD_GSV5:    ssrch_nxt = `SS_ASM2;                   /* rd, addr, set gsv            */
      `DD_UNK1:    ssrch_nxt = `SS_ASM1;                   /*     addr, set unk            */
      `DD_ASM1:    ssrch_nxt = `SS_ASM2;                   /* rd, wr_sb                    */
      `DD_ASM2: begin
        casex ({rx_idle, rx_full, rx_data})
          10'h0xx: ssrch_nxt = `SS_ASM2;
          10'h10A: ssrch_nxt = `SS_RDWT;                   /* <LF>                         */
          10'h124: ssrch_nxt = `SS_HD1;                    /* $                            */
          10'h2xx: ssrch_nxt = `SS_EXIT;
          10'h3xx: ssrch_nxt = `SS_RDEX;
          default: ssrch_nxt = `SS_ASM1;
          endcase
        end
      `DD_ESC1:    ssrch_nxt = `SS_ESC2;                   /* rd, wr_fifo, bcnt=512        */
      `DD_ESC2: begin
        case ({rx_idle, rx_full})
          2'b00:   ssrch_nxt = `SS_ESC2;
          2'b10:   ssrch_nxt = `SS_EXIT;
          2'b11:   ssrch_nxt = `SS_RDEX;
          default: ssrch_nxt = `SS_ESC3;
          endcase
        end
      `DD_ESC3:    ssrch_nxt = `SS_ESC2;                   /* rd, wr_fifo                  */
      `DD_RDEX:    ssrch_nxt = `SS_EXIT;                   /* rd                           */
      `DD_EXIT:    ssrch_nxt = `SS_IDLE;                   /*     release                  */
      default:     ssrch_nxt = `SS_IDLE;
      endcase
    end

  always @ (posedge clk or posedge pwr_upb) begin
    if (pwr_upb) ssrch_reg <= `SS_RESET;
    else         ssrch_reg <= ssrch_nxt;
    end

  /*****************************************************************************************/
  /* string buffer write-side control                                                  clk */
  /*****************************************************************************************/
  assign ld_sbwa  = ssrch_reg[`ID_RMC5] || ssrch_reg[`ID_VTG5] || ssrch_reg[`ID_GGA5] ||
                    ssrch_reg[`ID_GLL5] || ssrch_reg[`ID_GSA5] || ssrch_reg[`ID_GSV5] ||
                    ssrch_reg[`ID_UNK1];
  assign sbw_wr   = ssrch_reg[`ID_ASM1] && !rx_limit;

  assign sb00_wr  = sbwsel_reg[0] && (sbwadd_reg[10:9] == 2'b00) && sbw_wr;
  assign sb01_wr  = sbwsel_reg[0] && (sbwadd_reg[10:9] == 2'b01) && sbw_wr;
  assign sb02_wr  = sbwsel_reg[0] && (sbwadd_reg[10:9] == 2'b10) && sbw_wr;
  assign sb10_wr  = sbwsel_reg[1] && (sbwadd_reg[10:9] == 2'b00) && sbw_wr;
  assign sb11_wr  = sbwsel_reg[1] && (sbwadd_reg[10:9] == 2'b01) && sbw_wr;
  assign sb12_wr  = sbwsel_reg[1] && (sbwadd_reg[10:9] == 2'b10) && sbw_wr;

  always @ (ssrch_reg or gsa_reg or gsv_reg or unk_reg) begin
    casex (ssrch_reg) //synopsys parallel_case
      `DD_RMC5:    sbwadd_nxt = `RMCADD;                   /* RMC                          */
      `DD_VTG5:    sbwadd_nxt = `VTGADD;                   /* VTG                          */
      `DD_GLL5:    sbwadd_nxt = `GLLADD;                   /* GLL                          */
      `DD_GGA5:    sbwadd_nxt = `GGAADD;                   /* GGA                          */
      `DD_GSA5: begin
        case (gsa_reg)
          2'b00:   sbwadd_nxt = `GSV1ADD;                  /* GSA #1                       */
          2'b01:   sbwadd_nxt = `GSV2ADD;                  /* GSA #2                       */
          default: sbwadd_nxt = `INVADD;                   /* GSA >2                       */
          endcase
        end
      `DD_GSV5: begin
        case (gsv_reg)
          2'b00:   sbwadd_nxt = `GSV1ADD;                  /* GSV #1                       */
          2'b01:   sbwadd_nxt = `GSV2ADD;                  /* GSV #2                       */
          2'b10:   sbwadd_nxt = `GSV3ADD;                  /* GSV #3                       */
          default: sbwadd_nxt = `GSV4ADD;                  /* GSV >3                       */
          endcase
        end
      `DD_UNK1: begin
        case (unk_reg)
          2'b00:   sbwadd_nxt = `UNK1ADD;                  /* UNK #1                       */
          2'b01:   sbwadd_nxt = `UNK2ADD;                  /* UNK #2                       */
          default: sbwadd_nxt = `INVADD;                   /* UNK >2                       */
          endcase
        end
      default:     sbwadd_nxt = `INVADD;
      endcase
    end

  always @ (posedge clk or posedge pwr_upb) begin
    if                (pwr_upb) sbwadd_reg <= 11'h0;
    else if (ld_sbwa || sbw_wr) sbwadd_reg <= (ld_sbwa) ? {sbwadd_nxt, 7'h0} :
                                                           sbwadd_reg + 1'b1;
    end

  /*****************************************************************************************/
  /* multi-string counters (saturating) and string identifiers                         clk */
  /*****************************************************************************************/
  assign clr_ncnt   = ssrch_reg[`ID_RMC5];
  assign clr_nmea   = ssrch_reg[`ID_HD1];
  assign inc_gsa    = rx_gsa_reg && nmea_end;
  assign inc_gsv    = rx_gsv_reg && nmea_end;
  assign inc_unk    = rx_unk_reg && nmea_end;
  assign nmea_end   = ssrch_reg[`ID_ASM1] && ((rx_data == 8'h0a) || (rx_data == 8'h24));
  assign set_rx_gsa = ssrch_reg[`ID_GSA5];
  assign set_rx_gsv = ssrch_reg[`ID_GSV5];
  assign set_rx_rmc = ssrch_reg[`ID_RMC5];
  assign set_rx_unk = ssrch_reg[`ID_UNK1];

  always @ (posedge clk or posedge pwr_upb) begin
    if (pwr_upb) begin
      gsa_reg    <= 2'b0;
      gsv_reg    <= 2'b0;
      unk_reg    <= 2'b0;
      rx_gsa_reg <= 1'b0;
      rx_gsv_reg <= 1'b0;
      rx_rmc_reg <= 1'b0;
      rx_unk_reg <= 1'b0;
      end
    else begin
      if (clr_ncnt || inc_gsa) gsa_reg <= (clr_ncnt) ? 2'b00 : (gsa_reg + ~&gsa_reg);
      if (clr_ncnt || inc_gsv) gsv_reg <= (clr_ncnt) ? 2'b00 : (gsv_reg + ~&gsv_reg);
      if (clr_ncnt || inc_unk) unk_reg <= (clr_ncnt) ? 2'b00 : (unk_reg + ~&unk_reg);
      rx_gsa_reg <= !clr_nmea && (set_rx_gsa || rx_gsa_reg);
      rx_gsv_reg <= !clr_nmea && (set_rx_gsv || rx_gsv_reg);
      rx_rmc_reg <= !clr_nmea && (set_rx_rmc || rx_rmc_reg);
      rx_unk_reg <= !clr_nmea && (set_rx_unk || rx_unk_reg);
      end
    end

  /*****************************************************************************************/
  /* string parse for GSA and RMC valid                                                clk */
  /*****************************************************************************************/
  assign str_comma = ssrch_reg[`ID_ASM1] && (rx_data == 8'h2C);
  assign gsa_mode2 = (rx_gsa_reg && rx_read && (strtrk_reg == 2'b10)) || rxmode_clr;
  assign rmc_valid =  rx_rmc_reg && rx_read && (strtrk_reg == 2'b10);
  assign ld_strtrk = set_rx_gsa || set_rx_rmc || str_comma || gsa_mode2 || rmc_valid;

  always @ (posedge clk or posedge pwr_upb) begin
    if        (pwr_upb) strtrk_reg <= 2'b00;
    else if (ld_strtrk) strtrk_reg <= (set_rx_gsa || set_rx_rmc) ? 2'b00 :
                                                               (strtrk_reg + ~&strtrk_reg);
    end

  always @ (posedge clk or posedge pwr_upb) begin
    if (pwr_upb) begin
      mode2_reg  <= 3'b0;
      rmcval_reg <= 1'b0;
      end
    else begin
      if (gsa_mode2) mode2_reg  <= (rxmode_clr)       ? 3'b000 :
                                   (rx_data == 8'h33) ? 3'b100 :
                                   (rx_data == 8'h32) ? 3'b010 : 3'b001;
      if (rmc_valid) rmcval_reg <= (rx_data == 8'h41);
      end
    end

  /*****************************************************************************************/
  /* gnss talker id latch (P, N, L)                                                    clk */
  /*****************************************************************************************/
  always @ (posedge clk or posedge pwr_upb) begin
    if (pwr_upb) begin
      talkid0_reg <= 3'b00;
      talkid1_reg <= 3'b00;
      end
    else if (ssrch_reg[`ID_HD5]) begin
      if (sbwsel_reg[0]) talkid0_reg <= {~|rx_data[2:1], &rx_data[2:1],
                                          (rx_data[2] && !rx_data[1])};
      if (sbwsel_reg[1]) talkid1_reg <= {~|rx_data[2:1], &rx_data[2:1],
                                          (rx_data[2] && !rx_data[1])};
      end
    end

  assign talk_id = (sbrsel_reg[1]) ? talkid1_reg : 
                   (sbrsel_reg[0]) ? talkid0_reg : 3'h0;

  /*****************************************************************************************/
  /* string valid bits for both buffers                                                clk */
  /*****************************************************************************************/
  assign set_sb0_inv = sbwsel_reg[0] && sbwclm_reg;
  assign set_sb1_inv = sbwsel_reg[1] && sbwclm_reg;
  assign set_sb0val  = sbwsel_reg[0] && nmea_end;
  assign set_sb1val  = sbwsel_reg[1] && nmea_end;

  assign set_sb0_val[0]  = set_sb0val && (sbwadd_reg[10:7] == `RMCADD);
  assign set_sb0_val[1]  = set_sb0val && (sbwadd_reg[10:7] == `VTGADD);
  assign set_sb0_val[2]  = set_sb0val && (sbwadd_reg[10:7] == `GLLADD);
  assign set_sb0_val[3]  = set_sb0val && (sbwadd_reg[10:7] == `GGAADD);
  assign set_sb0_val[4]  = set_sb0val && (sbwadd_reg[10:7] == `UNK1ADD);
  assign set_sb0_val[5]  = set_sb0val && (sbwadd_reg[10:7] == `UNK2ADD);
  assign set_sb0_val[6]  = set_sb0val && (sbwadd_reg[10:7] == `GSA1ADD);
  assign set_sb0_val[7]  = set_sb0val && (sbwadd_reg[10:7] == `GSA2ADD);
  assign set_sb0_val[8]  = set_sb0val && (sbwadd_reg[10:7] == `GSV1ADD);
  assign set_sb0_val[9]  = set_sb0val && (sbwadd_reg[10:7] == `GSV2ADD);
  assign set_sb0_val[10] = set_sb0val && (sbwadd_reg[10:7] == `GSV3ADD);
  assign set_sb0_val[11] = set_sb0val && (sbwadd_reg[10:7] == `GSV4ADD);
  assign set_sb1_val[0]  = set_sb1val && (sbwadd_reg[10:7] == `RMCADD);
  assign set_sb1_val[1]  = set_sb1val && (sbwadd_reg[10:7] == `VTGADD);
  assign set_sb1_val[2]  = set_sb1val && (sbwadd_reg[10:7] == `GLLADD);
  assign set_sb1_val[3]  = set_sb1val && (sbwadd_reg[10:7] == `GGAADD);
  assign set_sb1_val[4]  = set_sb1val && (sbwadd_reg[10:7] == `UNK1ADD);
  assign set_sb1_val[5]  = set_sb1val && (sbwadd_reg[10:7] == `UNK2ADD);
  assign set_sb1_val[6]  = set_sb1val && (sbwadd_reg[10:7] == `GSA1ADD);
  assign set_sb1_val[7]  = set_sb1val && (sbwadd_reg[10:7] == `GSA2ADD);
  assign set_sb1_val[8]  = set_sb1val && (sbwadd_reg[10:7] == `GSV1ADD);
  assign set_sb1_val[9]  = set_sb1val && (sbwadd_reg[10:7] == `GSV2ADD);
  assign set_sb1_val[10] = set_sb1val && (sbwadd_reg[10:7] == `GSV3ADD);
  assign set_sb1_val[11] = set_sb1val && (sbwadd_reg[10:7] == `GSV4ADD);

  always @ (posedge clk or posedge pwr_upb) begin
    if (pwr_upb) begin
      sb0val_reg <= 12'b0;
      sb1val_reg <= 12'h0;
      end
    else begin
      sb0val_reg <= {12{set_sb0_inv}} & (set_sb0_val | sb0val_reg);
      sb1val_reg <= {12{set_sb1_inv}} & (set_sb1_val | sb1val_reg);
      end
    end

  assign sbr_valid = (sbrsel_reg[1]) ? sb1val_reg : 
                     (sbrsel_reg[0]) ? sb0val_reg : 12'h0;

  /*****************************************************************************************/
  /* string buffer claim/release state machines                                        clk */
  /*****************************************************************************************/
  assign sbwr_clm   = ssrch_reg[`ID_RMC5];
  assign sbwr_rel   = ssrch_reg[`ID_EXIT];
  assign ld_sblast  = sbwr_clm || sbwr_rel;
  assign ld_sbrsel  = sbrd_clm || sbrd_rel;
  assign ld_sbwsel  = sbwr_clm || sbwr_rel;
  assign sblast_nxt = (sblast_reg || sbwsel_reg[1]) && !sbwsel_reg[0];

  always @ (sblast_reg or sbwsel_reg or sbrd_clm or sbwr_rel or sbwr_clm) begin
    casex ({sblast_reg, sbwsel_reg, sbrd_clm, sbwr_rel, sbwr_clm})
      6'b000100: sbrsel_nxt = 2'b01;                       /* claim cases                  */
      6'bx01100: sbrsel_nxt = 2'b10;
      6'bx10100: sbrsel_nxt = 2'b01;
      6'b100100: sbrsel_nxt = 2'b10;
      6'b000110: sbrsel_nxt = 2'b01;
      6'bx01110: sbrsel_nxt = 2'b01;
      6'b010110: sbrsel_nxt = 2'b10;
      6'b100110: sbrsel_nxt = 2'b10;
      6'b110110: sbrsel_nxt = 2'b10;
      6'b000101: sbrsel_nxt = 2'b01;
      6'bx01101: sbrsel_nxt = 2'b01;
      6'b010101: sbrsel_nxt = 2'b10;
      6'b100101: sbrsel_nxt = 2'b10;
      6'b110101: sbrsel_nxt = 2'b10; 
      default:   sbrsel_nxt = 2'b00;                       /* non-claim cases              */ 
      endcase
    end

  always @ (sblast_reg or sbrsel_reg or sbwsel_reg or sbrd_clm or sbrd_rel or sbwr_clm) begin
    casex ({sblast_reg, sbrsel_reg, sbwsel_reg, sbrd_clm, sbrd_rel, sbwr_clm})
      8'b0000x001: sbwsel_nxt = 2'b10;                     /* claim cases                  */
      8'b00010001: sbwsel_nxt = 2'b01;
      8'b00100001: sbwsel_nxt = 2'b10;
      8'b00110001: sbwsel_nxt = 2'b01;
      8'b01000001: sbwsel_nxt = 2'b01;
      8'b01001001: sbwsel_nxt = 2'b10;
      8'b10000001: sbwsel_nxt = 2'b01;
      8'b10001001: sbwsel_nxt = 2'b10;
      8'b10010001: sbwsel_nxt = 2'b01;
      8'b10100001: sbwsel_nxt = 2'b10;
      8'b10110001: sbwsel_nxt = 2'b01;
      8'b11000001: sbwsel_nxt = 2'b01;
      8'b11001001: sbwsel_nxt = 2'b10;
      8'b00xxx011: sbwsel_nxt = 2'b10;
      8'b01xxx011: sbwsel_nxt = 2'b01;
      8'b1x0xx011: sbwsel_nxt = 2'b01;
      8'b101xx011: sbwsel_nxt = 2'b10;
      8'b00xxx101: sbwsel_nxt = 2'b10;
      8'b01xxx101: sbwsel_nxt = 2'b01;
      8'b1x0xx101: sbwsel_nxt = 2'b01;
      8'b101xx101: sbwsel_nxt = 2'b10;
      default:     sbwsel_nxt = 2'b00;                     /* non-claim cases              */ 
      endcase
    end

  always @ (posedge clk or posedge pwr_upb) begin
    if (pwr_upb) begin
      sbwclm_reg <= 1'b0;
      sblast_reg <= 1'b0;
      sbrsel_reg <= 2'b0;
      sbwsel_reg <= 2'b0;
      end
    else begin
      sbwclm_reg <= sbwr_clm;
      if (ld_sblast) sblast_reg <= sblast_nxt;
      if (ld_sbrsel) sbrsel_reg <= sbrsel_nxt;
      if (ld_sbwsel) sbwsel_reg <= sbwsel_nxt;
      end
    end

  endmodule





