/*******************************************************************************************/
/**                                                                                       **/
/** COPYRIGHT (C) 2019,  Systemyde International Corporation, ALL RIGHTS RESERVED         **/
/**                                                                                       **/
/** gnss top level                                                    Rev 0.0  09/20/2019 **/
/**                                                                                       **/
/*******************************************************************************************/
module gnss_top (data_drv, data_out, debug_out, fi_drv, fi_out, isa_drv, isa_out, spi_csn,
                 spi_so, data_bus, debug_ctl, isa_bus, osc_in, ph1, ph2, pwr_up, pwo,
                 spi_sck, spi_si, sync);

  input         data_bus;      /* memory/peripheral data bus                          bus  */
  input         isa_bus;       /* instruction address/data bus                        bus  */
  input         osc_in;        /* 32768Hz clock input                               32khz  */
  input         ph1;           /* clock phase 1                                       bus  */
  input         ph2;           /* clock phase 2                                       bus  */
  input         pwr_up;        /* power on reset                                      por  */
  input         pwo;           /* power on                                            bus  */
  input         spi_sck;       /* spi serial clock: pulldown                          ffc  */
  input         spi_si;        /* spi serial input: rx from gnss receiver             ffc  */
  input         sync;          /* synchronization                                     bus  */
  input   [2:0] debug_ctl;     /* debug select signals                                dbg  */

  output        data_drv;      /* memory/peripheral data bus enable                   bus  */
  output        data_out;      /* memory/peripheral data bus                          bus  */
  output        fi_drv;        /* flag bus enable                                     bus  */
  output        fi_out;        /* flag bus                                            bus  */
  output        isa_drv;       /* instruction address/data bus enable                 bus  */
  output        isa_out;       /* instruction address/data bus                        bus  */
  output        spi_csn;       /* spi chip select: gnss enable                        ffc  */
  output        spi_so;        /* spi serial output: tx to gnss receiver              ffc  */
  output  [8:0] debug_out;     /* debug output signals                                dbg  */

  /*****************************************************************************************/
  /* signal declarations                                                                   */
  /*****************************************************************************************/
  wire        data_drv;                                    /* data bus drive enable        */
  wire        data_out;                                    /* memory/peripheral data bus   */
  wire        fi_drv;                                      /* flag bus drive enable        */
  wire        fi_out;                                      /* flag bus data                */
  wire        isa_drv;                                     /* isa_bus drive                */
  wire        isa_drv_g;                                   /* isa_bus drive (gnss)         */
  wire        isa_drv_m;                                   /* isa_bus drive (memory)       */
  wire        isa_out;                                     /* isa_bus data                 */
  wire        isa_out_g;                                   /* isa_bus data (gnss)          */
  wire        isa_out_m;                                   /* isa_bus data (memory)        */
  wire        spi_csn;                                     /* spi chip select              */
  wire        spi_so;                                      /* spi serial output            */

  wire        enabl_rd;                                    /* rom read enable              */
  wire        enabl_wr;                                    /* rom write enable             */
  wire        gnss_en;                                     /* gnss receiver enable         */
  wire        osc_en;                                      /* oscillator enable            */
  wire  [3:0] page_addr;                                   /* rom page address             */

  wire        clk;                                         /* clk (2.4576MHz)              */
  wire        ph1_buf;                                     /* buffered ph1                 */
  wire        ph2_buf;                                     /* buffered ph2                 */
  wire        pwob;                                        /* power on (inverted)          */
  wire        pwr_upb;                                     /* pwr_up (inverted)            */

  wire        oscdiv_max;                                  /* hfosc divider max count      */
  wire        oscdiv_mid;                                  /* hfosc divider mid count      */
  wire        oscdiv_qua;                                  /* hfosc divider 1/4 count      */
  wire        syn_32k;                                     /* osc_in synced to clk48m      */

  reg         clkgen_reg   /* synthesis syn_preserve=1 */; /* 2.4576MHz clock              */
  reg         hfoscq_reg;                                  /* hfosc divider quadrant       */
  reg   [1:0] clkph_reg    /* synthesis syn_preserve=1 */; /* clkgen_reg/4                 */
  reg   [4:0] clkdiv_reg;                                  /* clk gen divider              */
  reg   [4:0] hfosch_lat, hfoscl_lat;                      /* hfosc latched div count      */
  reg   [4:0] hfosch_reg, hfoscl_reg;                      /* hfosc div count              */
  reg   [6:0] hfosc_div;                                   /* hfosc divider                */

  /*****************************************************************************************/
  /* clock domain crossing                                                                 */
  /*****************************************************************************************/
  reg   [2:0] osc_cdc_reg  /* synthesis syn_preserve=1 */; /* 32768 hz                     */

  /*****************************************************************************************/
  /* debug-specific                                                                        */
  /*****************************************************************************************/
  wire  [8:0] debug_out;                                   /* debug output signals         */

  /*****************************************************************************************/
  /* fpga-specific                                                                         */
  /*****************************************************************************************/
`ifdef SIM_VERSION

  reg         hf_osc;                                      /* fpga hf oscillator output    */

  assign      ph1_buf = ph1;
  assign      ph2_buf = ph2;
  assign      clk     = clkgen_reg;
  assign      pwob    = !pwo;
  assign      pwr_upb = !pwr_up;

  initial begin
    hf_osc <= 1'b0;
    end

  always begin
    #11 hf_osc <= !hf_osc;
    #10 hf_osc <= !hf_osc;
    end

`else

  wire        hf_osc;                                      /* fpga hf oscillator output    */

  SB_GB  PH1     ( .USER_SIGNAL_TO_GLOBAL_BUFFER(ph1),
                   .GLOBAL_BUFFER_OUTPUT(ph1_buf) );

  SB_GB  PH2     ( .USER_SIGNAL_TO_GLOBAL_BUFFER(ph2),
                   .GLOBAL_BUFFER_OUTPUT(ph2_buf) );

  SB_GB  PWO     ( .USER_SIGNAL_TO_GLOBAL_BUFFER(!pwo),
                   .GLOBAL_BUFFER_OUTPUT(pwob) );

  SB_GB  PWR_UP  ( .USER_SIGNAL_TO_GLOBAL_BUFFER(!pwr_up),
                   .GLOBAL_BUFFER_OUTPUT(pwr_upb) );

  SB_GB  CLK_BUF ( .USER_SIGNAL_TO_GLOBAL_BUFFER(clkgen_reg),
                   .GLOBAL_BUFFER_OUTPUT(clk) );

  SB_HFOSC  FOSC ( .CLKHFEN(osc_en), .CLKHFPU(osc_en),
                   .CLKHF(hf_osc) /* synthesis ROUTE_THROUGH_FABRIC = 0 */ );

`endif

  /*****************************************************************************************/
  /* generate master serial clock: 75 x 32768hz = 2.4576MHz = 64 x 38.4k            hf_osc */
  /*****************************************************************************************/
  assign oscdiv_max = (hfosc_div == 7'h4a);
  assign oscdiv_mid = (hfosc_div == 7'h25);
  assign oscdiv_qua = (hfosc_div == 7'h12) || (hfosc_div == 7'h37) ||
                       oscdiv_max || oscdiv_mid;

  always @ (posedge hf_osc or posedge pwr_upb) begin
    if (pwr_upb) begin
      osc_cdc_reg <= 3'b0;
      hfosc_div   <= 7'h0;
      hfosch_reg  <= 5'h0;
      hfoscl_reg  <= 5'h0;
      hfoscq_reg  <= 1'b0;
      hfosch_lat  <= 5'h0;
      hfoscl_lat  <= 5'h0;
      clkdiv_reg  <= 5'h0;
      clkph_reg   <= 2'b0;
      clkgen_reg  <= 1'b0;
      end
    else begin
      osc_cdc_reg <= {osc_in, osc_cdc_reg[2:1]};
      hfosc_div <= (syn_32k || oscdiv_max) ? 7'h0 : (hfosc_div + 1'b1);
      if (syn_32k || oscdiv_max) hfosch_reg <= (syn_32k) ? 5'h1f : (hfosch_reg + 1'b1);
      if (syn_32k || oscdiv_mid) hfoscl_reg <= (syn_32k) ? 5'h1f : (hfoscl_reg + 1'b1);
      if (syn_32k || oscdiv_qua) hfoscq_reg <= !syn_32k && !oscdiv_max && !oscdiv_mid;
      if (syn_32k) begin
        hfosch_lat <= hfosch_reg;
        hfoscl_lat <= hfoscl_reg;
        end
      clkdiv_reg <= (~|clkdiv_reg &&                   clkph_reg[0]) ?  hfosch_lat :
                    (~|clkdiv_reg && !clkph_reg[1] && !clkph_reg[0]) ?  hfoscl_lat :
                    (~|clkdiv_reg &&  clkph_reg[1] && !clkph_reg[0]) ? (hfoscl_lat + 1'b1) : 
                                                                       (clkdiv_reg - 1'b1);
      if (~|clkdiv_reg) clkph_reg <= (clkph_reg + 1'b1);
      clkgen_reg   <= |clkdiv_reg[4:3];
      end
    end

  assign syn_32k    = osc_cdc_reg[1] && !osc_cdc_reg[0];

  /*****************************************************************************************/
  /* isa bus output                                                                        */
  /*****************************************************************************************/
  assign isa_drv  = isa_drv_g || isa_drv_m;
  assign isa_out  = isa_out_g || isa_out_m;

  /*****************************************************************************************/
  /* gnss core                                                                             */
  /*****************************************************************************************/
  gnss_fpga CORE  ( .data_drv(data_drv), .data_out(data_out), .debug_out(debug_out),
                    .enabl_rd(enabl_rd), .enabl_wr(enabl_wr), .fi_drv(fi_drv),
                    .fi_out(fi_out), .gnss_en(gnss_en), .isa_drv(isa_drv_g),
                    .isa_out(isa_out_g), .osc_en(osc_en), .page_addr(page_addr),
                    .tx_dout(spi_so), .clk(clk), .data_bus(data_bus), .debug_ctl(debug_ctl),
                    .isa_bus(isa_bus), .ph1(ph1_buf), .ph2(ph2_buf), .pwr_upb(pwr_upb),
                    .pwob(pwob), .rx_din(spi_si), .sync(sync) );

  /*****************************************************************************************/
  /* rom (instruction memory)                                                              */
  /*****************************************************************************************/
`define GNSS

  imem_fpga IMEM  ( .isa_drv(isa_drv_m), .isa_out(isa_out_m), .data_bus(data_bus),
                    .enabl_rd(enabl_rd), .enabl_wr(enabl_wr), .isa_bus(isa_bus),
                    .page_addr(page_addr), .ph1(ph1_buf), .ph2(ph2_buf), .pwob(pwob),
                    .sync(sync) );

  /*****************************************************************************************/
  /* ffc signal placeholder                                                                */
  /*****************************************************************************************/
  assign spi_csn = gnss_en || spi_sck;

  endmodule





