/*******************************************************************************************/
/**                                                                                       **/
/** COPYRIGHT (C) 2018, Systemyde International Corporation, ALL RIGHTS RESERVED          **/
/**                                                                                       **/
/** writable rom module                                               Rev 0.0  08/02/2018 **/
/**                                                                                       **/
/*******************************************************************************************/
module imem_fpga (isa_drv, isa_out, data_bus, enabl_rd, enabl_wr, isa_bus, page_addr, ph1,
                  ph2, pwob, sync);

  input        isa_bus;        /* instruction address/data bus                        bus  */
  input        data_bus;       /* data bus                                            bus  */
  input        enabl_rd;       /* read enable                                         ctl  */
  input        enabl_wr;       /* write enable                                        ctl  */
  input        ph1;            /* clock phase 1                                       GB   */
  input        ph2;            /* clock phase 2                                       GB   */
  input        pwob;            /* power on (inverted)                                GB   */
  input        sync;           /* synchronization                                     bus  */
  input  [3:0] page_addr;      /* rom page address                                    ctl  */

  output       isa_drv;        /* instruction address/data bus                        bus  */
  output       isa_out;        /* instruction address/data bus                        bus  */

  /*****************************************************************************************/
  /* signal declarations                                                                   */
  /*****************************************************************************************/
  wire        isa_drv;                                     /* isa output enable            */
  wire        isa_out;                                     /* instruction out              */
  wire        ld_cycle;                                    /* update machine state         */
  wire        page_sel;                                    /* rom page selected for rd     */
  wire        te_addr;                                     /* time enable: address         */
  wire        te_dec;                                      /* time enable: addr decode     */
  wire        te_inst;                                     /* time enable: instruction     */
  wire        te_waddr;                                    /* time enable: wr addr         */
  wire        te_wdata;                                    /* time enable: wr data         */
  wire        te_wr;                                       /* time enable: wr strobe       */
  wire        wpage_sel;                                   /* rom page selected for wr     */
  wire  [1:0] rom00_d;                                     /* rom low half bits {1:0]      */
  wire  [1:0] rom01_d;                                     /* rom low half bits {3:2]      */
  wire  [1:0] rom02_d;                                     /* rom low half bits {5:4]      */
  wire  [1:0] rom03_d;                                     /* rom low half bits {7:6]      */
  wire  [1:0] rom04_d;                                     /* rom low half bits {9:8]      */
  wire  [1:0] rom10_d;                                     /* rom hi  half bits {1:0]      */
  wire  [1:0] rom11_d;                                     /* rom hi  half bits {3:2]      */
  wire  [1:0] rom12_d;                                     /* rom hi  half bits {5:4]      */
  wire  [1:0] rom13_d;                                     /* rom hi  half bits {7:6]      */
  wire  [1:0] rom14_d;                                     /* rom hi  half bits {9:8]      */
  wire  [9:0] rd_data;                                     /* read data                    */

  reg         data_reg;                                    /* ph1 sampled data_bus signal  */
  reg         isa_reg;                                     /* ph1 sampled isa signal       */
  reg         pagerom_reg;                                 /* rom page decode              */
  reg         rom_0_rclke;                                 /* lower bank rd clk enable     */
  reg         rom_0_wr;                                    /* lower bank wr enable         */
  reg         rom_1_rclke;                                 /* upper bank rd clk enable     */
  reg         rom_1_wr;                                    /* upper bank wr enable         */
  reg         rom_rd;                                      /* read enable                  */
  reg         state_0, state_1, state_2, state_3;          /* machine state                */
  reg         state_4, state_5, state_6;
  reg         state_8, state_9, state_a, state_b;
  reg         state_c, state_d, state_e;
  reg         state_w;                                     /* wake state                   */
  reg         sync_reg;                                    /* sampled sync                 */
  reg         syndet_reg;                                  /* sync the machine state       */
  reg         wr_inst;                                     /* write inst reg               */
  reg         wrom_reg;                                    /* wrom inst                    */
  reg   [3:0] cycle_reg;                                   /* clock cycle                  */
  reg   [3:0] syncnt_reg;                                  /* sync detector                */
  reg   [9:0] inst_reg;                                    /* instruction shifter          */
  reg   [9:0] winst_reg;                                   /* latched instruction          */
  reg  [11:0] wdata_reg;                                   /* latched write data           */
  reg  [15:0] addr_reg;                                    /* latched isa address          */
  reg  [15:0] waddr_reg;                                   /* latched write address        */

  /*****************************************************************************************/
  /* output signals                                                                        */
  /*****************************************************************************************/
  assign isa_drv = pagerom_reg && te_inst && !ph2;
  assign isa_out = inst_reg[0] && !ph2;

  /*****************************************************************************************/
  /* sample bus signals                                                                    */
  /*****************************************************************************************/
  always @ (negedge ph1 or posedge pwob) begin
    if (pwob) begin
      data_reg <= 1'b0;
      isa_reg  <= 1'b0;
      sync_reg <= 1'b0;
      end
    else begin
      data_reg <= data_bus;
      isa_reg  <= isa_bus;
      sync_reg <= sync;
      end
    end

  /*****************************************************************************************/
  /* bus synchronization                                                                   */
  /*****************************************************************************************/
  always @ (posedge ph2 or posedge pwob) begin
    if (pwob) begin
      syncnt_reg <= 4'b0000;
      syndet_reg <= 1'b0;
      end
    else begin
      syncnt_reg <= (sync_reg) ? (syncnt_reg + 1'b1) : 4'b0000;
      syndet_reg <= !sync_reg && ~|(syncnt_reg ^ 4'b1010);
      end
    end

  /*****************************************************************************************/
  /* bus state machine                                                                     */
  /*****************************************************************************************/
  /*                                                                                       */
  /*   state_x         eeeecccc88880000111122225555aaaa4444999933336666ddddbbbb            */
  /*   cycle_reg       12481248124812481248124812481248124812481248124812481248            */
  /*   ld_cycle        ...1...1...1...1...1...1...1...1...1...1...1...1...1...1            */
  /*   te_addr         ..............1111111111111111..........................            */
  /*   te_inst (sync)  ............................................1111111111..            */
  /*   te_dec          ...............................1........................            */
  /*   te_waddr        ............1111111111111111............................            */
  /*   te_wdat         111111111111............................................            */
  /*   te_wr           .............................1..........................            */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge ph2 or posedge pwob) begin
    if (pwob) cycle_reg <= 4'b0000;
    else      cycle_reg <= {( !cycle_reg[3]   && cycle_reg[2] && !syndet_reg),
                            (~|cycle_reg[3:2] && cycle_reg[1] && !syndet_reg),
                            (~|cycle_reg[3:1] && !syndet_reg),
                            (cycle_reg[3] || syndet_reg)};
    end

  always @ (posedge ph2 or posedge pwob) begin
    if (pwob) begin
      state_w <= 1'b0;
      state_e <= 1'b0;
      state_c <= 1'b0;
      state_8 <= 1'b0;
      state_0 <= 1'b0;
      state_1 <= 1'b0;
      state_2 <= 1'b0;
      state_5 <= 1'b0;
      state_a <= 1'b0;
      state_4 <= 1'b0;
      state_9 <= 1'b0;
      state_3 <= 1'b0;
      state_6 <= 1'b0;
      state_d <= 1'b0;
      state_b <= 1'b0;
      end
    else if (ld_cycle) begin
      state_w <= 1'b1;
      state_e <= state_b ||  syndet_reg || !state_w;
      state_c <= state_e && !syndet_reg;
      state_8 <= state_c && !syndet_reg;
      state_0 <= state_8 && !syndet_reg;
      state_1 <= state_0 && !syndet_reg;
      state_2 <= state_1 && !syndet_reg;
      state_5 <= state_2 && !syndet_reg;
      state_a <= state_5 && !syndet_reg;
      state_4 <= state_a && !syndet_reg;
      state_9 <= state_4 && !syndet_reg;
      state_3 <= state_9 && !syndet_reg;
      state_6 <= state_3 && !syndet_reg;
      state_d <= state_6 && !syndet_reg;
      state_b <= state_d && !syndet_reg;
      end
    end

  assign ld_cycle = cycle_reg[3];
  assign te_addr  = (state_0 && |cycle_reg[3:2]) || state_1 || state_2 || state_5 ||
                    (state_a && |cycle_reg[1:0]);
  assign te_dec   = state_a && cycle_reg[3];
  assign te_inst  = state_6 || state_d || (state_b && |cycle_reg[1:0]);
  assign te_waddr = state_0 || state_1 || state_2 || state_5;
  assign te_wdata = state_e || state_c || state_8;
  assign te_wr    = state_a && cycle_reg[0];

  /*****************************************************************************************/
  /* address sample                                                                        */
  /*****************************************************************************************/
  assign page_sel = enabl_rd && (addr_reg[15:12] == page_addr);

  always @ (posedge ph2 or posedge pwob) begin
    if (pwob) begin
      addr_reg    <= 16'h0000;
      pagerom_reg <= 1'b0;
      end
    else begin
      if (te_addr) addr_reg    <= {isa_reg, addr_reg[15:1]};
      if (te_dec)  pagerom_reg <= page_sel;
      end
    end

  /*****************************************************************************************/
  /* wrom inst/addr/data sample                                                            */
  /*****************************************************************************************/
  assign wpage_sel = enabl_wr && (waddr_reg[15:12] == page_addr);

  always @ (posedge ph2 or posedge pwob) begin
    if (pwob) begin
      waddr_reg <= 16'h0000;
      wdata_reg <= 12'h000;
      winst_reg <= 10'h000;
      wrom_reg  <= 1'b0;
      end
    else begin
      if (te_waddr) waddr_reg <= {data_reg, waddr_reg[15:1]};
      if (te_wdata) wdata_reg <= {data_reg, wdata_reg[11:1]};
      if (te_inst)  winst_reg <= {isa_reg,  winst_reg[9:1]};
      if (ld_cycle && state_b) wrom_reg <= (winst_reg == 10'h040);

      end
    end

  /*****************************************************************************************/
  /* write data machine                                                                    */
  /*****************************************************************************************/
  always @ (posedge ph2 or posedge pwob) begin
    if (pwob) begin
      rom_0_wr    <= 1'b0;
      rom_1_wr    <= 1'b0;
      end
    else begin
      rom_0_wr <= te_wr && wrom_reg && wpage_sel && !waddr_reg[11];
      rom_1_wr <= te_wr && wrom_reg && wpage_sel &&  waddr_reg[11];
      end
    end

  /*****************************************************************************************/
  /* read data machine                                                                     */
  /*****************************************************************************************/
  always @ (posedge ph2 or posedge pwob) begin
    if (pwob) begin
      rom_0_rclke <= 1'b0;
      rom_1_rclke <= 1'b0;
      rom_rd      <= 1'b0;
      wr_inst     <= 1'b0;
      end
    else begin
      rom_0_rclke <= te_dec && page_sel && !addr_reg[11];
      rom_1_rclke <= te_dec && page_sel &&  addr_reg[11];
      rom_rd      <= te_dec && page_sel;
      wr_inst     <= rom_rd;
      end
    end

  /*****************************************************************************************/
  /* instruction output register                                                           */
  /*****************************************************************************************/
  assign rd_data = (addr_reg[11]) ? {rom14_d, rom13_d, rom12_d, rom11_d, rom10_d} :
                                    {rom04_d, rom03_d, rom02_d, rom01_d, rom00_d};

  always @ (posedge ph2 or posedge pwob) begin
    if                    (pwob) inst_reg <= 10'h000;
    else if (wr_inst || te_inst) inst_reg <= (wr_inst) ? rd_data : {1'b0, inst_reg[9:1]};
    end

  /*****************************************************************************************/
  /* rom                                                                                   */
  /*****************************************************************************************/
`ifdef SIM_VERSION

  ram_2048x2 RAML_10    ( .RADDR(addr_reg[10:0]), .RCLK(ph2), .RCLKE(rom_0_rclke),
                          .RE(rom_rd), .RDATA(rom00_d), .WADDR(waddr_reg[10:0]), .WCLK(ph2),
                          .WCLKE(rom_0_wr), .WE(rom_0_wr), .WDATA(wdata_reg[1:0]) );

  ram_2048x2 RAML_32    ( .RADDR(addr_reg[10:0]), .RCLK(ph2), .RCLKE(rom_0_rclke),
                          .RE(rom_rd), .RDATA(rom01_d), .WADDR(waddr_reg[10:0]), .WCLK(ph2),
                          .WCLKE(rom_0_wr), .WE(rom_0_wr), .WDATA(wdata_reg[3:2]) );

  ram_2048x2 RAML_54    ( .RADDR(addr_reg[10:0]), .RCLK(ph2), .RCLKE(rom_0_rclke),
                          .RE(rom_rd), .RDATA(rom02_d), .WADDR(waddr_reg[10:0]), .WCLK(ph2),
                          .WCLKE(rom_0_wr), .WE(rom_0_wr), .WDATA(wdata_reg[5:4]) );

  ram_2048x2 RAML_76    ( .RADDR(addr_reg[10:0]), .RCLK(ph2), .RCLKE(rom_0_rclke),
                          .RE(rom_rd), .RDATA(rom03_d), .WADDR(waddr_reg[10:0]), .WCLK(ph2),
                          .WCLKE(rom_0_wr), .WE(rom_0_wr), .WDATA(wdata_reg[7:6]) );

  ram_2048x2 RAML_98    ( .RADDR(addr_reg[10:0]), .RCLK(ph2), .RCLKE(rom_0_rclke),
                          .RE(rom_rd), .RDATA(rom04_d), .WADDR(waddr_reg[10:0]), .WCLK(ph2),
                          .WCLKE(rom_0_wr), .WE(rom_0_wr), .WDATA(wdata_reg[9:8]) );

  ram_2048x2 RAMH_10    ( .RADDR(addr_reg[10:0]), .RCLK(ph2), .RCLKE(rom_1_rclke),
                          .RE(rom_rd), .RDATA(rom10_d), .WADDR(waddr_reg[10:0]), .WCLK(ph2),
                          .WCLKE(rom_1_wr), .WE(rom_1_wr), .WDATA(wdata_reg[1:0]) );

  ram_2048x2 RAMH_32    ( .RADDR(addr_reg[10:0]), .RCLK(ph2), .RCLKE(rom_1_rclke),
                          .RE(rom_rd), .RDATA(rom11_d), .WADDR(waddr_reg[10:0]), .WCLK(ph2),
                          .WCLKE(rom_1_wr), .WE(rom_1_wr), .WDATA(wdata_reg[3:2]) );

  ram_2048x2 RAMH_54    ( .RADDR(addr_reg[10:0]), .RCLK(ph2), .RCLKE(rom_1_rclke),
                          .RE(rom_rd), .RDATA(rom12_d), .WADDR(waddr_reg[10:0]), .WCLK(ph2),
                          .WCLKE(rom_1_wr), .WE(rom_1_wr), .WDATA(wdata_reg[5:4]) );

  ram_2048x2 RAMH_76    ( .RADDR(addr_reg[10:0]), .RCLK(ph2), .RCLKE(rom_1_rclke),
                          .RE(rom_rd), .RDATA(rom13_d), .WADDR(waddr_reg[10:0]), .WCLK(ph2),
                          .WCLKE(rom_1_wr), .WE(rom_1_wr), .WDATA(wdata_reg[7:6]) );

  ram_2048x2 RAMH_98    ( .RADDR(addr_reg[10:0]), .RCLK(ph2), .RCLKE(rom_1_rclke),
                          .RE(rom_rd), .RDATA(rom14_d), .WADDR(waddr_reg[10:0]), .WCLK(ph2),
                          .WCLKE(rom_1_wr), .WE(rom_1_wr), .WDATA(wdata_reg[9:8]) );
`else

  SB_RAM2048x2 RAML_10  ( .RADDR(addr_reg[10:0]), .RCLK(ph2), .RCLKE(rom_0_rclke),
                          .RE(rom_rd), .RDATA(rom00_d), .WADDR(waddr_reg[10:0]), .WCLK(ph2),
                          .WCLKE(rom_0_wr), .WE(rom_0_wr), .WDATA(wdata_reg[1:0]) );

  SB_RAM2048x2 RAML_32  ( .RADDR(addr_reg[10:0]), .RCLK(ph2), .RCLKE(rom_0_rclke),
                          .RE(rom_rd), .RDATA(rom01_d), .WADDR(waddr_reg[10:0]), .WCLK(ph2),
                          .WCLKE(rom_0_wr), .WE(rom_0_wr), .WDATA(wdata_reg[3:2]) );

  SB_RAM2048x2 RAML_54  ( .RADDR(addr_reg[10:0]), .RCLK(ph2), .RCLKE(rom_0_rclke),
                          .RE(rom_rd), .RDATA(rom02_d), .WADDR(waddr_reg[10:0]), .WCLK(ph2),
                          .WCLKE(rom_0_wr), .WE(rom_0_wr), .WDATA(wdata_reg[5:4]) );

  SB_RAM2048x2 RAML_76  ( .RADDR(addr_reg[10:0]), .RCLK(ph2), .RCLKE(rom_0_rclke),
                          .RE(rom_rd), .RDATA(rom03_d), .WADDR(waddr_reg[10:0]), .WCLK(ph2),
                          .WCLKE(rom_0_wr), .WE(rom_0_wr), .WDATA(wdata_reg[7:6]) );

  SB_RAM2048x2 RAML_98  ( .RADDR(addr_reg[10:0]), .RCLK(ph2), .RCLKE(rom_0_rclke),
                          .RE(rom_rd), .RDATA(rom04_d), .WADDR(waddr_reg[10:0]), .WCLK(ph2),
                          .WCLKE(rom_0_wr), .WE(rom_0_wr), .WDATA(wdata_reg[9:8]) );

  SB_RAM2048x2 RAMH_10  ( .RADDR(addr_reg[10:0]), .RCLK(ph2), .RCLKE(rom_1_rclke),
                          .RE(rom_rd), .RDATA(rom10_d), .WADDR(waddr_reg[10:0]), .WCLK(ph2),
                          .WCLKE(rom_1_wr), .WE(rom_1_wr), .WDATA(wdata_reg[1:0]) );

  SB_RAM2048x2 RAMH_32  ( .RADDR(addr_reg[10:0]), .RCLK(ph2), .RCLKE(rom_1_rclke),
                          .RE(rom_rd), .RDATA(rom11_d), .WADDR(waddr_reg[10:0]), .WCLK(ph2),
                          .WCLKE(rom_1_wr), .WE(rom_1_wr), .WDATA(wdata_reg[3:2]) );

  SB_RAM2048x2 RAMH_54  ( .RADDR(addr_reg[10:0]), .RCLK(ph2), .RCLKE(rom_1_rclke),
                          .RE(rom_rd), .RDATA(rom12_d), .WADDR(waddr_reg[10:0]), .WCLK(ph2),
                          .WCLKE(rom_1_wr), .WE(rom_1_wr), .WDATA(wdata_reg[5:4]) );

  SB_RAM2048x2 RAMH_76  ( .RADDR(addr_reg[10:0]), .RCLK(ph2), .RCLKE(rom_1_rclke),
                          .RE(rom_rd), .RDATA(rom13_d), .WADDR(waddr_reg[10:0]), .WCLK(ph2),
                          .WCLKE(rom_1_wr), .WE(rom_1_wr), .WDATA(wdata_reg[7:6]) );

  SB_RAM2048x2 RAMH_98  ( .RADDR(addr_reg[10:0]), .RCLK(ph2), .RCLKE(rom_1_rclke),
                          .RE(rom_rd), .RDATA(rom14_d), .WADDR(waddr_reg[10:0]), .WCLK(ph2),
                          .WCLKE(rom_1_wr), .WE(rom_1_wr), .WDATA(wdata_reg[9:8]) );
`endif

  /*****************************************************************************************/
  /* initialize rom                                                                        */
  /*****************************************************************************************/
`ifdef TIMER
  `include "tmr_code.v"
`endif

`ifdef XFNS
  `include "xfn_code.v"
`endif

`ifdef GNSS
  `include "gns_code.v"
`endif

  endmodule





